# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.function.view.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Function Views."""

# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.widgets import RAMSTKModuleView, RAMSTKPanel, RAMSTKWorkView

# RAMSTK Local Imports
from . import FunctionGeneralDataPanel, FunctionTreePanel


class FunctionModuleView(RAMSTKModuleView):
    """Display Function attribute data in the RAMSTK Module Book.

    The Function Module View displays all the Functions associated with the
    connected RAMSTK Program in a flat list.  The attributes of a Function
    Module View are:

    :cvar _tag: the name of the module.

    :ivar _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "function"
    _tablabel: str = "Function"
    _tabtooltip: str = _("Displays the functional hierarchy for the selected Revision.")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Function Module View.

        :param configuration: the RAMSTK Configuration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.
        self._dic_icons["tab"] = (
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/function.png"
        )

        # Initialize private list attributes.
        self._lst_callbacks.insert(1, super().do_request_insert_child)
        self._lst_icons[0] = "insert_sibling"
        self._lst_icons.insert(1, "insert_child")
        self._lst_mnu_labels = [
            _("Add Sibling Function"),
            _("Add Child Function"),
            _("Delete Selected Function"),
            _("Save Selected Function"),
            _("Save All Functions"),
        ]
        self._lst_tooltips = [
            _("Add a new sibling function."),
            _("Add a new child function."),
            _("Delete the currently selected function."),
            _("Save changes to the currently selected function."),
            _("Save changes to all functions."),
        ]

        # Initialize private scalar attributes.
        self._pnlPanel = FunctionTreePanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, f"selected_{self._tag}")

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the stakeholder input's record ID.

        :param attributes: the attributes dict for the selected function.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["revision_id"] = attributes["revision_id"]
        self.dic_pkeys["function_id"] = attributes["function_id"]
        self.dic_pkeys["parent_id"] = attributes["parent_id"]
        self.dic_pkeys["record_id"] = attributes["function_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the function module view.

        :return: None
        """
        super().make_ui()

        self._pnlPanel.tvwTreeView.dic_handler_id[
            "button-press"
        ] = self._pnlPanel.tvwTreeView.connect(
            "button_press_event", super().on_button_press
        )


class FunctionWorkView(RAMSTKWorkView):
    """Display general Function attribute data in the RAMSTK Work Book.

    The Function Work View displays all the general data attributes for the
    selected Function. The attributes of a Function General Data Work View are:

    :cvar str _tag: the name of the module.
    :cvar str _tablabel: the text to display on the tab's label.
    :cvar str _tabtooltip: the text to display as the tab's tooltip.

    :ivar list _lst_callbacks: the list of callback methods for the view's
        toolbar buttons and pop-up menu.  The methods are listed in the order
        they appear on the toolbar and pop-up menu.
    :ivar list _lst_icons: the list of icons for the view's toolbar buttons
        and pop-up menu.  The icons are listed in the order they appear on the
        toolbar and pop-up menu.
    :ivar list _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar list _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dict class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "function"
    _tablabel = _("General\nData")
    _tabtooltip = _("Displays general information for the selected Function.")

    # Define public dict class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Function Work View general data page.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_mnu_labels = [
            _("Save Selected Function"),
            _("Save All Functions"),
        ]
        self._lst_tooltips = [
            _("Save changes to the currently selected function."),
            _("Save changes to all functions."),
        ]

        # Initialize private scalar attributes.
        self._pnlGeneralData: RAMSTKPanel = FunctionGeneralDataPanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, "selected_function")

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the stakeholder input's record ID.

        :param attributes: the attributes dict for the selected stakeholder
            input.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["parent_id"] = attributes["parent_id"]
        self.dic_pkeys["record_id"] = attributes["function_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the Function General Data tab.

        :return: None
        :rtype: None
        """
        super().do_make_layout()

        self.pack_end(self._pnlGeneralData, True, True, 0)
        self.show_all()



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel
from ramstk.views.gtk3.widgets import RAMSTKTextView as RAMSTKTextView
from ramstk.views.gtk3.widgets import RAMSTKTreePanel as RAMSTKTreePanel

class RevisionTreePanel(RAMSTKTreePanel):
    _select_msg: str
    _tag: str
    _title: Any
    _type: str
    dic_attribute_widget_map: Any
    def __init__(self) -> None: ...
    def _on_module_switch(self, module: str = ...) -> None: ...
    _record_id: Any
    def _on_row_change(self, selection: Gtk.TreeSelection) -> None: ...

class RevisionGeneralDataPanel(RAMSTKFixedPanel):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: Any
    _type: str
    txtCode: Any
    txtName: Any
    txtRemarks: Any
    dic_attribute_widget_map: Any
    def __init__(self) -> None: ...



# Standard Library Imports
from typing import List

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager as RAMSTKLogManager
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKListView as RAMSTKListView
from ramstk.views.gtk3.widgets import RAMSTKPanel

# RAMSTK Local Imports
from . import UsageProfileTreePanel as UsageProfileTreePanel

class UsageProfileListView(RAMSTKListView):
    _tag: str
    _tablabel: str
    _tabtooltip: str
    _lst_col_order: List[int]
    _lst_mnu_labels: List[str]
    _lst_tooltips: List[str]
    _pnlPanel: RAMSTKPanel
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    def _do_request_delete(self, __button: Gtk.ToolButton) -> None: ...
    def _do_request_insert_child(self, __button: Gtk.ToolButton) -> None: ...
    def _do_request_insert_sibling(self, __button: Gtk.ToolButton) -> None: ...
    def __make_ui(self) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.revision.view.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Revision Views."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub  # type: ignore

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import (
    RAMSTKMessageDialog,
    RAMSTKModuleView,
    RAMSTKPanel,
    RAMSTKWorkView,
)

# RAMSTK Local Imports
from . import RevisionGeneralDataPanel, RevisionTreePanel


class RevisionModuleView(RAMSTKModuleView):
    """Display Revision attribute data in the RAMSTK Module Book.

    The Revision Module View displays all the Revisions associated with the
    connected RAMSTK Program in a flat list.  The attributes of a Revision
    Module View are:

    :cvar _tag: the name of the module.

    :ivar _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "revision"
    _tablabel: str = "Revision"
    _tabtooltip: str = _("Displays the list of Revisions for the open RAMSTK Project.")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Revision Module View.

        :param configuration: the RAMSTK Configuration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.
        self._dic_icons["tab"] = (
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/revision.png"
        )

        # Initialize private list attributes.
        self._lst_mnu_labels = [
            _("Add Revision"),
            _("Delete Selected Revision"),
            _("Save Selected Revision"),
            _("Save All Revisions"),
        ]
        self._lst_tooltips = [
            _("Add a new revision."),
            _("Remove the currently selected revision."),
            _("Save changes to the currently selected revision."),
            _("Save changes to all revisions."),
        ]

        # Initialize private scalar attributes.
        self._pnlPanel = RevisionTreePanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, "selected_revision")

    def do_request_delete(self, __button: Gtk.ToolButton) -> None:
        """Request to delete selected record from the RAMSTKRevision table.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        """
        _parent = self.get_parent().get_parent().get_parent().get_parent().get_parent()
        _prompt = _(
            "You are about to delete Revision {0:d} and all "
            "data associated with it.  Is this really what "
            "you want to do?"
        ).format(self._revision_id)
        _dialog = RAMSTKMessageDialog(parent=_parent)
        _dialog.do_set_message(_prompt)
        _dialog.do_set_message_type("question")

        if _dialog.do_run() == Gtk.ResponseType.YES:
            super().do_set_cursor_busy()
            pub.sendMessage("request_delete_revision", node_id=self._revision_id)

        _dialog.do_destroy()

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the Revision's record ID.

        :param attributes: the attributes dict for the selected Revision.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["record_id"] = attributes["revision_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the revision module view.

        :return: None
        """
        super().make_ui()

        self._pnlPanel.tvwTreeView.dic_handler_id[
            "button-press"
        ] = self._pnlPanel.tvwTreeView.connect(
            "button_press_event", super().on_button_press
        )


class RevisionWorkView(RAMSTKWorkView):
    """Display general Revision attribute data in the RAMSTK Work Book.

    The Revision Work View displays all the general data attributes for the
    selected Revision. The attributes of a Revision General Data Work View are:

    :cvar str _tag: the name of the module.
    :cvar str _tablabel: the text to display on the tab's label.
    :cvar str _tabtooltip: the text to display as the tab's tooltip.

    :ivar list _lst_callbacks: the list of callback methods for the view's
        toolbar buttons and pop-up menu.  The methods are listed in the order
        they appear on the toolbar and pop-up menu.
    :ivar list _lst_icons: the list of icons for the view's toolbar buttons
        and pop-up menu.  The icons are listed in the order they appear on the
        toolbar and pop-up menu.
    :ivar list _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar list _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dict class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "revision"
    _tablabel = _("General\nData")
    _tabtooltip = _("Displays general information for the selected Revision")

    # Define public dict class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Revision Work View general data page.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_tooltips: List[str] = [
            _("Save changes to the currently selected Revision."),
            _("Save changes to all Revisions."),
        ]

        # Initialize private scalar attributes.
        self._pnlGeneralData: RAMSTKPanel = RevisionGeneralDataPanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, "selected_revision")

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the Revision's record ID.

        :param attributes: the attributes dict for the selected Revision.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["record_id"] = attributes["revision_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the Revision General Data tab.

        :return: None
        :rtype: None
        """
        super().do_make_layout()

        self.pack_end(self._pnlGeneralData, True, True, 0)
        self.show_all()



# -*- coding: utf-8 -*-
#
#       ramstk.exceptions.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTK Exceptions Module."""


class RAMSTKError(Exception):
    """Basic exception for errors raised by RAMSTK."""

    def __init__(self, msg: str = "") -> None:
        """Initialize the basic RAMSTK exception.

        :keyword str msg: the message to display to the user when this
            exception is raised.
        """
        if msg == "":
            # Set some default useless error message
            msg = "An error occured with RAMSTK."

        super().__init__(msg)


class DataAccessError(RAMSTKError):
    """Exception raised when attempting to access non-existent data."""

    def __init__(self, msg: str) -> None:
        """Initialize DataAccessError instance.

        This exception is intended for use for non-existent data in the data
        access object and the treelib Tree()'s.  Use it to override the
        native exceptions.

        :param msg: the message to display to the user when this
            exception is raised.
        """
        super().__init__(msg=msg)

        self.msg = msg


class OutOfRangeError(RAMSTKError):
    """Exception raised when an input value is outside legal limits."""

    def __init__(self, msg: str) -> None:
        """Initialize OutOfRangeError instance.

        :param msg: the message to display to the user when this
            exception is raised.
        """
        super().__init__(msg=msg)

        self.msg = msg



# Standard Library Imports
from typing import Any, Dict, List

PART_COUNT_LAMBDA_B_DICT: Any
PART_COUNT_LAMBDA_B_LIST: Any
PART_COUNT_PI_Q: Dict[int, List[float]]
PART_COUNT_PI_Q_HF_DIODE: List[List[float]]
PART_STRESS_PI_Q: Dict[int, List[float]]
PART_STRESS_PI_Q_HF_DIODE: Dict[int, List[float]]
PI_T_DICT: Dict[int, List[float]]
PI_T_LIST: Dict[int, List[float]]
PI_T_SCALAR: Dict[int, float]
CASE_TEMPERATURE: Any
THETA_JC: Any
PI_C: Any
PI_E: Any
PI_M: Any

def calculate_application_factor(attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def calculate_electrical_stress_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def calculate_junction_temperature(attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def calculate_part_count(**attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def calculate_part_stress(**attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def calculate_part_stress_lambda_b(attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def calculate_power_rating_factor(attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def calculate_temperature_factor(attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def get_part_count_lambda_b(attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def get_part_count_quality_factor(attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def get_part_stress_quality_factor(attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def _get_section_6_1_electrical_stress_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_2_application_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_2_power_rating_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_3_application_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_3_electrical_stress_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_3_power_rating_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_4_application_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_6_electrical_stress_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_6_power_rating_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_7_application_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_8_application_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_10_electrical_stress_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...
def _get_section_6_13_application_factor(
    attributes: Dict[str, Any]
) -> Dict[str, Any]: ...



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.design_electric.components.miscellaneous.py is part of the
#       RAMSTK Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Miscellaneous Devices Input Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.widgets import RAMSTKComboBox, RAMSTKEntry, RAMSTKFixedPanel


class MiscDesignElectricInputPanel(RAMSTKFixedPanel):
    """Display Miscellaneous assessment input attribute data.

    The Miscellaneous hardware assessment input view displays all the
    assessment inputs for the selected miscellaneous hardware item.  This
    includes, currently, inputs for MIL-HDBK-217FN2.  The attributes of a
    Miscellaneous hardware assessment input view are:

    :ivar cmbApplication: select and display the application of the
        miscellaneous item (lamps only).
    :ivar cmbType: the type of miscellaneous item (filters only).
    :ivar txtFrequency: enter and display the operating frequency of the
        miscellaneous item (crystals only).
    :ivar txtUtilization: enter and display the utilization factor of the
        miscellaneous item (lamps only).
    """

    # Define private dict class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "selected_hardware"
    _tag: str = "design_electric"
    _title: str = _("Miscellaneous Device Design Inputs")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize instance of the Miscellaneous assessment input view."""
        super().__init__()

        # Initialize widgets.
        self.cmbApplication: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbQuality: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbType: RAMSTKComboBox = RAMSTKComboBox()
        self.txtFrequency: RAMSTKEntry = RAMSTKEntry()
        self.txtUtilization: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._duty_cycle: float = 100.0
        self._hazard_rate_method_id: int = 0
        self._quality_id: int = 0

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "quality_id": [
                32,
                self.cmbQuality,
                "changed",
                super().on_changed_combo,
                "wvw_editing_reliability",
                0,
                {
                    "tooltip": _("The quality level."),
                },
                _("Quality Level:"),
                "gint",
            ],
            "application_id": [
                2,
                self.cmbApplication,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The application of the lamp."),
                },
                _("Application:"),
                "gint",
            ],
            "type_id": [
                48,
                self.cmbType,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The type of electronic filter."),
                },
                _("Type:"),
                "gint",
            ],
            "frequency_operating": [
                17,
                self.txtFrequency,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The operating frequency of the crystal."),
                },
                _("Operating Frequency:"),
                "gfloat",
            ],
            "duty_cycle": [
                12,
                self.txtUtilization,
                "changed",
                super().on_changed_entry,
                "wvw_editing_hardware",
                100.0,
                {
                    "tooltip": _(
                        "The utilization factor (illuminate hours / equipment operate "
                        "hours) of the lamp."
                    ),
                },
                _("Utilization:"),
                "gfloat",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.category_id: int = 0
        self.subcategory_id: int = 0

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            self.do_load_comboboxes,
            "changed_subcategory",
        )
        pub.subscribe(
            self._do_set_hardware_attributes,
            "succeed_get_hardware_attributes",
        )
        pub.subscribe(
            self._do_set_reliability_attributes,
            "succeed_get_reliability_attributes",
        )

    def do_load_comboboxes(self, subcategory_id: int) -> None:
        """Load the miscellaneous assessment input RKTComboBox()s.

        :param subcategory_id: the subcategory ID of the selected miscellaneous device.
        :return: None
        :rtype: None
        """
        self.subcategory_id = subcategory_id

        # Load the quality level RAMSTKComboBox().
        self.cmbQuality.do_load_combo([["MIL-SPEC"], [_("Lower")]], signal="changed")

        # Load the application RAMSTKComboBox().
        self.cmbApplication.do_load_combo(
            [[_("Incandescent, AC")], [_("Incandescent, DC")]], signal="changed"
        )

        # Load the type RAMSTKComboBox().
        if self._hazard_rate_method_id == 1:
            self.cmbType.do_load_combo(
                [
                    [_("Ceramic-Ferrite")],
                    [_("Discrete LC Components")],
                    [_("Discrete LC and Crystal Components")],
                ],
                signal="changed",
            )
        elif self._hazard_rate_method_id == 2:
            self.cmbType.do_load_combo(
                [
                    [_("MIL-F-15733 Ceramic-Ferrite")],
                    [_("MIL-F-15733 Discrete LC Components")],
                    [_("MIL-F-18327 Discrete LC Components")],
                    [_("MIL-F-18327 Discrete LC and Crystal Components")],
                ],
                signal="changed",
            )

    def _do_set_hardware_attributes(self, attributes: Dict[str, Any]) -> None:
        """Set the attributes when the hardware attributes are retrieved.

        :param attributes: the dict of hardware attributes.
        :return: None
        :rtype: None
        """
        if attributes["hardware_id"] == self._record_id:
            self._duty_cycle = attributes["duty_cycle"]

    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None:
        """Set the attributes when the reliability attributes are retrieved.

        :param attributes: the dict of reliability attributes.
        :return: None
        :rtype: None
        """
        if attributes["hardware_id"] == self._record_id:
            self._hazard_rate_method_id = attributes["hazard_rate_method_id"]
            self._quality_id = attributes["quality_id"]

            self.cmbQuality.set_sensitive(True)
            self.cmbQuality.do_update(
                self._quality_id,
                signal="changed",
            )

            pub.sendMessage(
                f"request_get_{self._tag}_attributes",
                node_id=self._record_id,
            )

    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set widget sensitivity for the selected Miscellaneous item.

        :return: None
        :rtype: None
        """
        self.cmbApplication.set_sensitive(False)
        self.cmbType.set_sensitive(False)
        self.txtFrequency.set_sensitive(False)
        self.txtUtilization.set_sensitive(False)

        _dic_method = {
            1: self.__do_set_crystal_sensitive,
            2: self.__do_set_filter_sensitive,
            4: self.__do_set_lamp_sensitive,
        }
        try:
            _dic_method[self.subcategory_id](attributes)
        except KeyError:
            pass

    def __do_set_crystal_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set the widget sensitivity as needed for a Crystal.

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 2:
            self.txtFrequency.set_sensitive(True)
            self.txtFrequency.do_update(
                attributes["frequency_operating"],
                signal="changed",
            )

    def __do_set_filter_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set the widget sensitivity as needed for an electronic filter.

        :return: None
        :rtype: None
        """
        self.cmbType.set_sensitive(True)
        self.cmbType.do_update(
            attributes["type_id"],
            signal="changed",
        )

    def __do_set_lamp_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set the widget sensitivity as needed for a Lamp.

        :return: None
        :rtype: None
        """
        self.cmbApplication.set_sensitive(True)
        self.cmbApplication.do_update(
            attributes["application_id"],
            signal="changed",
        )

        if self._hazard_rate_method_id == 2:
            self.txtUtilization.set_sensitive(True)
            self.txtUtilization.do_update(
                self._duty_cycle,
                signal="changed",
            )



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.RAMSTKGroup.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKGroup Table Module."""

# Third Party Imports
from sqlalchemy import Column, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKGroup(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent the table ramstk_group in the RAMSTK Common database.

    This table shares a Many-to-One relationship with ramstk_user.
    """

    __defaults__ = {"description": "Group Description", "group_type": ""}
    __tablename__ = "ramstk_group"
    __table_args__ = {"extend_existing": True}

    group_id = Column(
        "fld_group_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    group_type = Column(
        "fld_group_type", String(256), default=__defaults__["group_type"]
    )

    def get_attributes(self):
        """Retrieve current values of the RAMSTKGroup data model attributes.

        :return: {workgroup_id, description, group_type} pairs
        :rtype: dict
        """
        _attributes = {
            "group_id": self.group_id,
            "description": self.description,
            "group_type": self.group_type,
        }

        return _attributes



# Standard Library Imports
from typing import Any

# Third Party Imports
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKModeRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any
    __tablename__: str
    __table_args__: Any
    revision_id: Any
    hardware_id: Any
    mode_id: Any
    critical_item: Any
    description: Any
    design_provisions: Any
    detection_method: Any
    effect_end: Any
    effect_local: Any
    effect_next: Any
    effect_probability: Any
    hazard_rate_source: Any
    isolation_method: Any
    mission: Any
    mission_phase: Any
    mode_criticality: Any
    mode_hazard_rate: Any
    mode_op_time: Any
    mode_probability: Any
    mode_ratio: Any
    operator_actions: Any
    other_indications: Any
    remarks: Any
    rpn_severity: Any
    rpn_severity_new: Any
    severity_class: Any
    single_point: Any
    type_id: Any
    mechanism: relationship
    is_mode: bool
    is_mechanism: bool
    is_cause: bool
    is_control: bool
    is_action: bool
    is_opload: bool
    is_opstress: bool
    is_testmethod: bool
    def get_attributes(self): ...



# -*- coding: utf-8 -*-
#
#       ramstk.utilities.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Utility functions for interacting with the RAMSTK application."""

# Standard Library Imports
import functools
import gettext
import os
import os.path
import sys
import warnings
from datetime import datetime
from typing import Any, Callable, List

# Third Party Imports
from dateutil.parser import parse

_ = gettext.gettext


def date_to_ordinal(date: str) -> int:
    """Convert date strings to ordinal dates for use in the database.

    :param date: the date string to convert.
    :return: ordinal representation of the passed date.
    :rtype: int
    """
    try:
        return parse(str(date)).toordinal()
    except (ValueError, TypeError):
        return parse("01/01/1970").toordinal()


def deprecated(func: Callable) -> Callable:
    """This is a decorator to mark functions as deprecated."""

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        warnings.simplefilter("always", DeprecationWarning)  # turn off filter
        warnings.warn(
            f"Call to deprecated function {func.__name__}.",
            category=DeprecationWarning,
            stacklevel=2,
        )
        warnings.simplefilter("default", DeprecationWarning)  # reset filter
        return func(*args, **kwargs)

    return new_func


def dir_exists(directory: str) -> bool:
    """Check if a directory exists.

    :param directory: a string representing the directory path to check
                          for.
    :return: False if the directory does not exist, True otherwise.
    :rtype: bool
    """
    return os.path.isdir(directory)


def file_exists(_file: str) -> bool:
    """Check if a file exists.

    :param _file: a string representing the filepath to check for.
    :return: True if the file exists, False otherwise.
    :rtype: bool
    """
    return os.path.isfile(_file)


def none_to_default(field: Any, default: Any) -> Any:
    """Convert None values into default values.

    :param field: the original value that may be None.
    :param default: the new, default, value.
    :return: field; the new value if field is None, the old value
        otherwise.
    :rtype: any
    """
    _return = field
    if field is None:
        _return = default

    return _return


def none_to_string(string: None) -> str:
    """Convert None types to an empty string.

    :param string: the string to convert.
    :return: string; the converted string.
    :rtype: str
    """
    _return = string
    if string is None or string == "None":
        _return = ""

    return _return


def ordinal_to_date(ordinal: int) -> str:
    """Convert ordinal dates to date strings in ISO-8601 format.

    Defaults to the current date if a bad value is passed as the argument.

    :param ordinal: the ordinal date to convert.
    :return: the ISO-8601 date representation of the passed ordinal.
    :rtype: str
    """
    try:
        return str(datetime.fromordinal(int(ordinal)).strftime("%Y-%m-%d"))
    except ValueError:
        ordinal = datetime.now().toordinal()
        return str(datetime.fromordinal(int(ordinal)).strftime("%Y-%m-%d"))


def split_string(string: str) -> List[str]:
    """Split a colon-delimited string into its constituent parts.

    :param string: the colon delimited string that needs to be split into
        a list.
    :return: _strlist
    :rtype: list of strings
    """
    _strlist = string.rsplit(":")

    return _strlist


def boolean_to_integer(boolean: bool) -> int:
    """Convert boolean representations of TRUE/FALSE to an integer value.

    :param bool boolean: the boolean to convert.
    :return: _result
    :rtype: int
    """
    _result = 0

    if boolean:
        _result = 1

    return _result


def integer_to_boolean(integer: int) -> bool:
    """Convert an integer to boolean value.

    Any value greater than zero is returned as True, all others are returned as
    False.

    :param integer: the integer to convert.
    :return: _result
    :rtype: bool
    :raise: TypeError if passed a string.
    """
    _result = False

    if integer > 0:
        _result = True

    return _result


def string_to_boolean(string: str) -> bool:
    """Convert string representations of TRUE/FALSE to an boolean value.

    :param string: the string to convert.
    :return: _result
    :rtype: bool
    """
    _result = False

    _string = str(string)

    if (
        _string.lower() == "true"
        or _string.lower() == "yes"
        or _string.lower() == "t"
        or _string.lower() == "y"
    ):
        _result = True

    return _result


def get_install_prefix() -> str:
    """Return the prefix that this code was installed into."""
    # Constants for this execution
    _path = os.path.abspath(__file__)
    _name = os.path.basename(os.path.dirname(_path))
    _this = os.path.basename(_path)

    # Rule set
    _rules: List[Any] = [
        # To match: /usr/lib[64]/pythonX.Y/site-packages/project/prefix.py
        # Or: /usr/local/lib[64]/pythonX.Y/dist-packages/project/prefix.py
        lambda x: x in ["lib64", "lib"],  # nosec
        lambda x: x == ("python%s" % sys.version[:3]),
        lambda x: x in ["site-packages", "dist-packages"],
        lambda x: x == _name,  # 'project'
        lambda x: x == _this,  # 'prefix.py'
    ]

    # Matching engine
    while _rules:
        (_path, _token) = os.path.split(_path)
        _rule = _rules.pop()
        # To account for the possibility python is using lib instead of lib64
        # on a 64-bit or multilib system.
        if not _rule(_token) and _token != "lib64":  # nosec
            _path = "/usr"

    return _path



# -*- coding: utf-8 -*-
#
#       ramstk.analyses.prediction.Lamp.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Lamp MIL-HDBK-217F Calculations Module."""

# Standard Library Imports
from typing import Any, Dict

PART_COUNT_LAMBDA_B = {
    1: [
        3.9,
        7.8,
        12.0,
        12.0,
        16.0,
        16.0,
        16.0,
        19.0,
        23.0,
        19.0,
        2.7,
        16.0,
        23.0,
        100.0,
    ],
    2: [
        13.0,
        26.0,
        38.0,
        38.0,
        51.0,
        51.0,
        51.0,
        64.0,
        77.0,
        64.0,
        9.0,
        51.0,
        77.0,
        350.0,
    ],
}
PI_E = [1.0, 2.0, 3.0, 3.0, 4.0, 4.0, 4.0, 5.0, 6.0, 5.0, 0.7, 4.0, 6.0, 27.0]


def calculate_part_count(**attributes: Dict[str, Any]) -> float:
    """Wrap get_part_count_lambda_b().

    This wrapper allows us to pass an attributes dict from a generic parts
    count function.

    :param attributes: the attributes for the connection being calculated.
    :return: _base_hr; the parts count base hazard rates.
    :rtype: float
    """
    return get_part_count_lambda_b(
        attributes["application_id"],  # type: ignore
        attributes["environment_active_id"],  # type: ignore
    )  # type: ignore


def calculate_part_stress(**attributes: Dict[str, Any]) -> Dict[str, Any]:
    """Calculate the part stress hazard rate for a lamp.

    This function calculates the MIL-HDBK-217F hazard rate using the part
    stress method.

    :return: attributes; the keyword argument (hardware attribute)
             dictionary with updated values.
    :rtype: dict
    """
    attributes["lambda_b"] = 0.074 * attributes["voltage_rated"] ** 1.29  # type: ignore

    # Determine the utilization factor (piU).
    if attributes["duty_cycle"] < 10.0:  # type: ignore
        attributes["piU"] = 0.1  # type: ignore
    elif 10.0 <= attributes["duty_cycle"] < 90.0:  # type: ignore
        attributes["piU"] = 0.72  # type: ignore
    else:
        attributes["piU"] = 1.0  # type: ignore

    # Determine the application factor (piA).
    attributes["piA"] = (
        3.3 if (attributes["application_id"]) - (1) else 1.0  # type: ignore
    )

    attributes["hazard_rate_active"] = (
        attributes["lambda_b"]  # type: ignore
        * attributes["piU"]
        * attributes["piA"]
        * attributes["piE"]
    )

    return attributes


def get_part_count_lambda_b(application_id: int, environment_active_id: int) -> float:
    """Retrieve the part count hazard rate for a lamp.

    This function calculates the MIL-HDBK-217F hazard rate using the parts
    count method.

    :param application_id: the lamp application identifier.
    :param environment_active_id: the operating environment identifier.
    :return: _base_hr; the base part count hazard rate.
    :rtype: float
    :raise: IndexError if passed an unknown active environment ID.
    :raise: KeyError if passed an unknown application ID.
    """
    return PART_COUNT_LAMBDA_B[application_id][environment_active_id - 1]



# Standard Library Imports
from typing import Any, Callable, Dict, List, Tuple, Union

# Third Party Imports
import treelib

# RAMSTK Package Imports
from ramstk.models import RAMSTKCauseRecord as RAMSTKCauseRecord
from ramstk.models import RAMSTKMechanismRecord as RAMSTKMechanismRecord
from ramstk.views.gtk3 import GdkPixbuf as GdkPixbuf
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKCheckButton as RAMSTKCheckButton
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel
from ramstk.views.gtk3.widgets import RAMSTKLabel as RAMSTKLabel
from ramstk.views.gtk3.widgets import RAMSTKTextView as RAMSTKTextView
from ramstk.views.gtk3.widgets import RAMSTKTreePanel as RAMSTKTreePanel

class FMEAMethodPanel(RAMSTKFixedPanel):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    chkCriticality: RAMSTKCheckButton
    chkRPN: RAMSTKCheckButton
    txtItemCriticality: RAMSTKTextView
    _on_edit_message: str
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def _do_load_item_criticality(self, item_criticality: Dict[str, float]) -> None: ...

class FMEATreePanel(RAMSTKTreePanel):
    _select_msg: str
    _tag: str
    _title: str
    _dic_mission_phases: Dict[str, List[str]]
    _dic_row_loader: Dict[str, Callable]
    _dic_visible_mask: Dict[str, List[str]]
    _lst_missions: List[str]
    _on_edit_message: str
    dic_attribute_widget_map: Dict[str, List[Any]]
    dic_icons: Dict[str, str]
    lst_action_category: List[str]
    lst_action_status: List[str]
    lst_control_types: List[str]
    lst_rpn_detection: List[str]
    lst_rpn_occurrence: List[str]
    lst_rpn_severity: List[str]
    lst_mode_probability: List[str]
    lst_severity_class: List[str]
    lst_users: List[str]
    def __init__(self) -> None: ...
    def do_load_comboboxes(self) -> None: ...
    def _on_mission_change(
        self, __combo: Gtk.CellRendererCombo, path: str, new_text: str
    ) -> None: ...
    def _on_row_change(self, selection: Gtk.TreeSelection) -> None: ...
    def __do_get_rpn_names(
        self, entity: Union[RAMSTKCauseRecord, RAMSTKMechanismRecord]
    ) -> Tuple[str, str, str, str]: ...
    def __do_load_action(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...
    def __do_load_action_category(self) -> None: ...
    def __do_load_action_status(self) -> None: ...
    def __do_load_cause(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...
    def __do_load_control(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...
    def __do_load_control_type(self) -> None: ...
    def __do_load_mechanism(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...
    def __do_load_missions(
        self, tree: treelib.Tree = ..., node_id: Any = ..., row: Gtk.TreeIter = ...
    ) -> None: ...
    def __do_load_mission_phases(self, mission: str) -> None: ...
    def __do_load_mode(self, node: treelib.Node, row: Gtk.TreeIter) -> Gtk.TreeIter: ...
    def __do_load_mode_probability(self) -> None: ...
    def __do_load_rpn_detection(self) -> None: ...
    def __do_load_rpn_occurrence(self) -> None: ...
    def __do_load_rpn_severity(self) -> None: ...
    def __do_load_severity_class(self) -> None: ...
    def __do_load_users(self) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.RAMSTKSubCategory.py is part of The RAMSTK
#       Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKSubCategory Table Module."""

# Third Party Imports
from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKSubCategory(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_subcategory in the RAMSTK Common database."""

    __defaults__ = {"description": "Subcategory Description"}
    __tablename__ = "ramstk_subcategory"
    __table_args__ = {"extend_existing": True}

    category_id = Column(
        "fld_category_id",
        Integer,
        ForeignKey("ramstk_category.fld_category_id"),
        nullable=False,
    )
    subcategory_id = Column(
        "fld_subcategory_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )

    # Define the relationships to other tables in the RAMSTK Program database.
    category = relationship(  # type: ignore
        "RAMSTKCategory", back_populates="subcategory"
    )
    mode = relationship(  # type: ignore
        "RAMSTKFailureMode",
        back_populates="subcategory",
        cascade="delete",
    )

    def get_attributes(self):
        """Retrieve current values of RAMSTKSubCategory data model attributes.

        :return: {category_id, subcategory, description} pairs.
        :rtype: dict
        """
        _attributes = {
            "category_id": self.category_id,
            "subcategory_id": self.subcategory_id,
            "description": self.description,
        }

        return _attributes



# Standard Library Imports
from typing import Dict, Union

# Third Party Imports
from sqlalchemy import UniqueConstraint
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKHardwareRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Dict[str, Union[float, int, str]]
    __tablename__: str
    __table_args__: UniqueConstraint
    revision_id: int
    hardware_id: int
    alt_part_number: str
    attachments: str
    cage_code: str
    category_id: int
    comp_ref_des: str
    cost: float
    cost_failure: float
    cost_hour: float
    cost_type_id: int
    description: str
    duty_cycle: float
    figure_number: str
    lcn: str
    level: int
    manufacturer_id: int
    mission_time: float
    name: str
    nsn: str
    page_number: str
    parent_id: int
    part: int
    part_number: str
    quantity: int
    ref_des: str
    remarks: str
    repairable: int
    specification_number: str
    subcategory_id: int
    tagged_part: int
    total_cost: float
    total_part_count: int
    total_power_dissipation: float
    year_of_manufacture: int
    allocation: relationship
    sia: relationship
    reliability: relationship
    milhdbkf: relationship
    nswc: relationship
    design_electric: relationship
    design_mechanic: relationship
    def get_attributes(self) -> Dict[str, Union[float, int, str]]: ...
    def do_calculate_total_cost(self) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.hardware.components.meter.py is part of the RAMSTK Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Meter Results Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.milhdbk217f import MilHdbk217FResultPanel
from ramstk.views.gtk3.widgets import RAMSTKEntry


class MeterMilHdbk217FResultPanel(MilHdbk217FResultPanel):
    """Display Meter assessment results attribute data in the RAMSTK Work Book.

    The Meter assessment result view displays all the assessment results
    for the selected meter.  This includes, currently, results for
    MIL-HDBK-217FN2 parts count and MIL-HDBK-217FN2 part stress methods.  The
    attributes of a meter assessment result view are:

    :ivar txtPiA: displays the application factor for the panel meter.
    :ivar txtPiF: displays the function factor for the panel meter.
    :ivar txtPiT: displays the temperature stress factor for the elapsed time
        meter.
    """

    # Define private class dict class attributes.
    _dic_part_stress: Dict[int, str] = {
        1: '<span foreground="blue">\u03BB<sub>p</sub> = '
        "\u03BB<sub>b</sub>\u03C0<sub>A</sub>\u03C0<sub>F</sub>\u03C0<sub>Q"
        "</sub>\u03C0<sub>E</sub></span>",
        2: '<span foreground="blue">\u03BB<sub>p</sub> = '
        "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>E</sub></span> ",
    }

    # Define private class list class attributes.

    # Define private scalar class attributes.
    _title: str = _("Meter MIL-HDBK-217F Results")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Meter assessment result view."""
        super().__init__()

        # Initialize widgets.
        self.txtPiA: RAMSTKEntry = RAMSTKEntry()
        self.txtPiF: RAMSTKEntry = RAMSTKEntry()
        self.txtPiT: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "hazard_rate_model": [
                13,
                self.lblModel,
                "",
                None,
                "",
                "",
                {
                    "tooltip": _(
                        "The assessment model used to calculate the meter hazard rate."
                    ),
                },
                "",
            ],
            "lambda_b": [
                23,
                self.txtLambdaB,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _("The base hazard rate for the meter."),
                },
                "\u03BB<sub>b</sub>:",
            ],
            "pi_q": [
                30,
                self.txtPiQ,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The quality factor for the meter."),
                },
                "\u03C0<sub>Q</sub>:",
            ],
            "pi_e": [
                19,
                self.txtPiE,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The environment factor for the meter."),
                },
                "\u03C0<sub>E</sub>:",
            ],
            "pi_a": [
                12,
                self.txtPiA,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The application factor for the meter."),
                },
                "\u03C0<sub>A</sub>:",
            ],
            "pi_f": [
                20,
                self.txtPiF,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The function factor for the meter."),
                },
                "\u03C0<sub>F</sub>:",
            ],
            "pi_t": [
                33,
                self.txtPiT,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The temperature stress factor for the elapsed time meter."
                    ),
                },
                "\u03C0<sub>T</sub>:",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        super().do_set_properties()
        super().do_make_panel()

        # Subscribe to PyPubSub messages.

    def _do_load_entries(self, attributes: Dict[str, Any]) -> None:
        """Load the meter assessment results page.

        :param attributes: the attributes dictionary for the selected
            Meter.
        :return: None
        :rtype: None
        """
        super().do_load_entries(attributes)

        self.txtPiA.set_sensitive(False)
        self.txtPiF.set_sensitive(False)
        self.txtPiT.set_sensitive(False)

        if self.category_id == 9 and self._hazard_rate_method_id == 2:
            self.txtPiA.do_update(str(self.fmt.format(attributes["piA"])))
            self.txtPiF.do_update(str(self.fmt.format(attributes["piF"])))
            self.txtPiT.do_update(str(self.fmt.format(attributes["piT"])))



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.hardware.components.capacitor.py is part of the
#       RAMSTK Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Capacitor Input Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.widgets import RAMSTKComboBox, RAMSTKEntry, RAMSTKFixedPanel


class CapacitorDesignElectricInputPanel(RAMSTKFixedPanel):
    """Display Capacitor assessment input attribute data.

    The Capacitor assessment input view displays all the assessment inputs for
    the selected capacitor.  This includes, currently, inputs for
    MIL-HDBK-217FN2 parts count and part stress analyses.  The attributes of a
    Capacitor assessment input view are:

    :cvar dict _dic_quality: dictionary of MIL-HDBK-217 capacitor quality
        levels.  Key is capacitor subcategory ID; values are lists of quality
        levels.
    :cvar dict _dic_specifications: dictionary of capacitor MIL-SPECs.  Key is
        capacitor subcategory ID; values are lists of specifications.
    :cvar dict _dic_styles: dictionary of capacitor styles defined in the
        MIL-SPECs.  Key is capacitor subcategory ID; values are lists of
        styles.

    :ivar list _lst_labels: list of label text to display for the capacitor
        MIL-HDBK-217 input parameters.
    :ivar _lst_widgets: the list of widgets to display in the panel.  These
        are listed in the order they should appear on the panel.

    :ivar cmbConfiguration: select and display the configuration of the
        capacitor.
    :ivar cmbConstruction: select and display the method of construction of the
        capacitor.
    :ivar cmbSpecification: select and display the governing specification of
        the capacitor.
    :ivar cmbStyle: select and display the style of the capacitor.
    :ivar txtCapacitance: enter and display the capacitance rating of the
        capacitor.
    :ivar txtESR: enter and display the equivalent series resistance.
    """

    # Define private dictionary class attributes.
    _dic_quality: Dict[int, List[Any]] = {
        1: [["MIL-SPEC"], [_("Lower")]],
        2: [["M"], [_("MIL-C-11693 Non-Established Reliability")], [_("Lower")]],
        3: [
            "S",
            "R",
            "P",
            "M",
            "L",
            [_("MIL-C-19978 Non-Established Reliability")],
            [_("Lower")],
        ],
        4: [
            "S",
            "R",
            "P",
            "M",
            "L",
            [_("MIL-C-18312 Non-Established Reliability")],
            [_("Lower")],
        ],
        5: ["S", "R", "P", "M", [_("Lower")]],
        6: ["S", "R", "P", "M", [_("Lower")]],
        7: [
            "T",
            "S",
            "R",
            "P",
            "M",
            "L",
            [_("MIL-C-5 Non-Established Reliability, Dipped")],
            [_("MIL-C-5 Non-Established Reliability, Molded")],
            [_("Lower")],
        ],
        8: [["MIL-C-10950"], [_("Lower")]],
        9: [
            "S",
            "R",
            "P",
            "M",
            "L",
            [_("MIL-C-11272 Non-Established Reliability")],
            [_("Lower")],
        ],
        10: [
            "S",
            "R",
            "P",
            "M",
            "L",
            [_("MIL-C-11015 Non-Established Reliability")],
            [_("Lower")],
        ],
        11: [
            "S",
            "R",
            "P",
            "M",
            [_("MIL-C-20 Non-Established Reliability")],
            [_("Lower")],
        ],
        12: ["D", "C", "S", "B", "R", "P", "M", "L", [_("Lower")]],
        13: [
            "S",
            "R",
            "P",
            "M",
            "L",
            [_("MIL-C-3965 Non-Established Reliability")],
            [_("Lower")],
        ],
        14: [
            "S",
            "R",
            "P",
            "M",
            [_("MIL-C-39018 Non-Established Reliability")],
            [_("Lower")],
        ],
        15: [["MIL-SPEC"], [_("Lower")]],
        16: [["MIL-SPEC"], [_("Lower")]],
        17: [["MIL-SPEC"], [_("Lower")]],
        18: [["MIL-SPEC"], [_("Lower")]],
        19: [["MIL-SPEC"], [_("Lower")]],
    }
    _dic_specifications: Dict[int, List[Any]] = {
        1: [["MIL-C-25"], ["MIL-C-12889"]],
        2: [["MIL-C-11693"]],
        3: [["MIL-C-14157"], ["MIL-C-19978"]],
        4: [["MIL-C-18312"], ["MIL-C-39022"]],
        5: [["MIL-C-55514"]],
        6: [["MIL-C-83421"]],
        7: [["MIL-C-5"], ["MIL-C-39001"]],
        8: [["MIL-C-10950"]],
        9: [["MIL-C-11272"], ["MIL-C-23269"]],
        10: [["MIL-C-11015"], ["MIL-C-39014"]],
        11: [["MIL-C-20"], ["MIL-C-55681"]],
        12: [["MIL-C-39003"]],
        13: [["MIL-C-3965"], ["MIL-C-39006"]],
        14: [["MIL-C-39018"]],
        15: [["MIL-C-62"]],
        16: [["MIL-C-81"]],
        17: [["MIL-C-14409"]],
        18: [["MIL-C-92"]],
        19: [["MIL-C-23183"]],
    }
    _dic_styles: Dict[int, List[Any]] = {
        1: [
            [
                ["CP4"],
                ["CP5"],
                ["CP8"],
                ["CP9"],
                ["CP10"],
                ["CP11"],
                ["CP12"],
                ["CP13"],
                ["CP25"],
                ["CP26"],
                ["CP27"],
                ["CP28"],
                ["CP29"],
                ["CP40"],
                ["CP41"],
                ["CP67"],
                ["CP69"],
                ["CP70"],
                ["CP72"],
                ["CP75"],
                ["CP76"],
                ["CP77"],
                ["CP78"],
                ["CP80"],
                ["CP81"],
                ["CP82"],
            ],
            [["CA"]],
        ],
        2: [
            [_("Characteristic E")],
            [_("Characteristic K")],
            [_("Characteristic P")],
            [_("Characteristic W")],
        ],
        3: [
            [["CPV07"], ["CPV09"], ["CPV17"]],
            [
                [_("Characteristic E")],
                [_("Characteristic F")],
                [_("Characteristic G")],
                [_("Characteristic K")],
                [_("Characteristic L")],
                [_("Characteristic M")],
                [_("Characteristic P")],
                [_("Characteristic Q")],
                [_("Characteristic S")],
                [_("Characteristic T")],
            ],
        ],
        4: [
            [[_("Characteristic N")], [_("Characteristic R")]],
            [
                [_("Characteristic 1")],
                [_("Characteristic 9")],
                [_("Characteristic 10")],
                [_("Characteristic 12")],
                [_("Characteristic 19")],
                [_("Characteristic 29")],
                [_("Characteristic 49")],
                [_("Characteristic 59")],
            ],
        ],
        5: [
            [_("Characteristic M")],
            [_("Characteristic N")],
            [_("Characteristic Q")],
            [_("Characteristic R")],
            [_("Characteristic S")],
        ],
        6: [["CRH"]],
        7: [
            [
                [_("Temperature Range M")],
                [_("Temperature Range N")],
                [_("Temperature Range O")],
                [_("Temperature Range P")],
            ],
            [[_("Temperature Range O")], [_("Temperature Range P")]],
        ],
        8: [["CB50"], [_("Other")]],
        9: [
            [[_("Temperature Range C")], [_("Temperature Range D")]],
            [[_("All")]],
        ],
        10: [
            [
                [_("Type A Rated Temperature")],
                [_("Type B Rated Temperature")],
                [_("Type C Rated Temperature")],
            ],
            [
                ["CKR05"],
                ["CKR06"],
                ["CKR07"],
                ["CKR08"],
                ["CKR09"],
                ["CKR10"],
                ["CKR11"],
                ["CKR12"],
                ["CKR13"],
                ["CKR14"],
                ["CKR15"],
                ["CKR16"],
                ["CKR17"],
                ["CKR18"],
                ["CKR19"],
                ["CKR48"],
                ["CKR64"],
                ["CKR72"],
                ["CKR73"],
                ["CKR74"],
            ],
        ],
        11: [
            [
                ["CC5"],
                ["CC6"],
                ["CC7"],
                ["CC8"],
                ["CC9"],
                ["CC13"],
                ["CC14"],
                ["CC15"],
                ["CC16"],
                ["CC17"],
                ["CC18"],
                ["CC19"],
                ["CC20"],
                ["CC21"],
                ["CC22"],
                ["CC25"],
                ["CC26"],
                ["CC27"],
                ["CC30"],
                ["CC31"],
                ["CC32"],
                ["CC33"],
                ["CC35"],
                ["CC36"],
                ["CC37"],
                ["CC45"],
                ["CC47"],
                ["CC50"],
                ["CC51"],
                ["CC52"],
                ["CC53"],
                ["CC54"],
                ["CC55"],
                ["CC56"],
                ["CC57"],
                ["CC75"],
                ["CC76"],
                ["CC77"],
                ["CC78"],
                ["CC79"],
                ["CC81"],
                ["CC82"],
                ["CC83"],
                ["CC85"],
                ["CC95"],
                ["CC96"],
                ["CC97"],
                ["CCR05"],
                ["CCR06"],
                ["CCR07"],
                ["CCR08"],
                ["CCR09"],
                ["CCR13"],
                ["CCR14"],
                ["CCR15"],
                ["CCR16"],
                ["CCR17"],
                ["CCR18"],
                ["CCR19"],
                ["CCR54"],
                ["CCR55"],
                ["CCR56"],
                ["CCR57"],
                ["CCR75"],
                ["CCR76"],
                ["CCR77"],
                ["CCR78"],
                ["CCR79"],
                ["CCR81"],
                ["CCR82"],
                ["CCR83"],
                ["CCR90"],
            ],
            [["CDR"]],
        ],
        12: [["CSR"]],
        13: [
            [
                ["CL10"],
                ["CL13"],
                ["CL14"],
                ["CL16"],
                ["CL17"],
                ["CL18"],
                ["CL24"],
                ["CL25"],
                ["CL26"],
                ["CL27"],
                ["CL30"],
                ["CL31"],
                ["CL32"],
                ["CL33"],
                ["CL34"],
                ["CL35"],
                ["CL36"],
                ["CL37"],
                ["CL40"],
                ["CL41"],
                ["CL42"],
                ["CL43"],
                ["CL46"],
                ["CL47"],
                ["CL48"],
                ["CL49"],
                ["CL50"],
                ["CL51"],
                ["CL52"],
                ["CL53"],
                ["CL54"],
                ["CL55"],
                ["CL56"],
                ["CL64"],
                ["CL65"],
                ["CL66"],
                ["CL67"],
                ["CL70"],
                ["CL71"],
                ["CL72"],
                ["CL73"],
            ],
            [["CLR"]],
        ],
        14: [[_("Style 16")], [_("Style 17")], [_("Style 71")], [_("All Others")]],
        15: [["CE"]],
        16: [
            ["CV11"],
            ["CV14"],
            ["CV21"],
            ["CV31"],
            ["CV32"],
            ["CV34"],
            ["CV35"],
            ["CV36"],
            ["CV40"],
            ["CV41"],
        ],
        17: [
            [_("Style G")],
            [_("Style H")],
            [_("Style J")],
            [_("Style L")],
            [_("Style Q")],
            [_("Style T")],
        ],
        18: [["CT"]],
        19: [
            ["CG20"],
            ["CG21"],
            ["CG30"],
            ["CG31"],
            ["CG32"],
            ["CG40"],
            ["CG41"],
            ["CG42"],
            ["CG43"],
            ["CG44"],
            ["CG50"],
            ["CG51"],
            ["CG60"],
            ["CG61"],
            ["CG62"],
            ["CG63"],
            ["CG64"],
            ["CG65"],
            ["CG66"],
            ["CG67"],
        ],
    }

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "selected_hardware"
    _tag: str = "design_electric"
    _title: str = _("Capacitor Design Inputs")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Capacitor assessment input view."""
        super().__init__()

        # Initialize widgets.
        self.cmbConfiguration: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbConstruction: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbQuality: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbSpecification: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbStyle: RAMSTKComboBox = RAMSTKComboBox()
        self.txtCapacitance: RAMSTKEntry = RAMSTKEntry()
        self.txtESR: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._hazard_rate_method_id: int = 0
        self._quality_id: int = 0

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map = {
            "quality_id": [
                32,
                self.cmbQuality,
                "changed",
                super().on_changed_combo,
                "wvw_editing_reliability",
                0,
                {
                    "tooltip": _("The quality level of the capacitor."),
                },
                _("Quality Level:"),
                "gint",
            ],
            "capacitance": [
                4,
                self.txtCapacitance,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _(
                        "The capacitance rating (in farads) of the capacitor."
                    ),
                },
                _("Capacitance (F):"),
                "gfloat",
            ],
            "specification_id": [
                36,
                self.cmbSpecification,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The governing specification for the capacitor."),
                },
                _("Specification:"),
                "gint",
            ],
            "type_id": [
                48,
                self.cmbStyle,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The style of the capacitor."),
                },
                _("Style:"),
                "gint",
            ],
            "configuration_id": [
                5,
                self.cmbConfiguration,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The configuration of the capacitor."),
                },
                _("Configuration:"),
                "gint",
            ],
            "construction_id": [
                6,
                self.cmbConstruction,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The method of construction of the capacitor."),
                },
                _("Construction:"),
                "gint",
            ],
            "resistance": [
                35,
                self.txtESR,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The equivalent series resistance of the capacitor."),
                },
                _("Equivalent Series Resistance (\u03A9):"),
                "gfloat",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.category_id: int = 0
        self.subcategory_id: int = 0

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.cmbSpecification.connect("changed", self._do_load_styles)

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            self.do_load_comboboxes,
            "changed_subcategory",
        )
        pub.subscribe(
            self._do_set_reliability_attributes,
            "succeed_get_reliability_attributes",
        )

    def do_load_comboboxes(self, subcategory_id: int) -> None:
        """Load the capacitor assessment input RAMSTKComboBox()s.

        :param subcategory_id: the subcategory ID of the selected capacitor.
        :return: None
        :rtype: None
        """
        self.subcategory_id = subcategory_id

        if self._hazard_rate_method_id == 1:  # MIL-HDBK-217F parts count.
            _quality: List[Any] = ["S", "R", "P", "M", "L", ["MIL-SPEC"], [_("Lower")]]
        else:
            try:
                _quality = self._dic_quality[self.subcategory_id]
            except KeyError:
                _quality = []

        self.cmbQuality.do_load_combo(_quality, signal="changed")

        try:
            _specification: List[Any] = self._dic_specifications[self.subcategory_id]
        except KeyError:
            _specification = []

        self.cmbSpecification.do_load_combo(_specification, signal="changed")

        self.cmbStyle.do_load_combo([], signal="changed")

        self.cmbConfiguration.do_load_combo(
            [[_("Fixed")], [_("Variable")]], signal="changed"
        )

        _construction: List[Any] = [
            [_("Slug, All Tantalum")],
            [_("Foil, Hermetic")],
            [_("Slug, Hermetic")],
            [_("Foil, Non-Hermetic")],
            [_("Slug, Non-Hermetic")],
        ]
        self.cmbConstruction.do_load_combo(_construction, signal="changed")

    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None:
        """Set the attributes when the reliability attributes are retrieved.

        :param attributes: the dict of reliability attributes.
        :return: None
        :rtype: None
        """
        if attributes["hardware_id"] == self._record_id:
            self._hazard_rate_method_id = attributes["hazard_rate_method_id"]
            self._quality_id = attributes["quality_id"]

            self.cmbQuality.set_sensitive(True)
            self.cmbQuality.do_update(
                self._quality_id,
                signal="changed",
            )

            pub.sendMessage(
                f"request_get_{self._tag}_attributes",
                node_id=self._record_id,
            )

    def _do_load_styles(self, combo: RAMSTKComboBox) -> None:
        """Load the style RAMSTKComboBox() when the specification changes.

        :param combo: the specification RAMSTKCombo() that called this method.
        :return: None
        :rtype: None
        """
        # If the capacitor specification changed, load the capacitor style
        # RAMSTKComboBox().
        try:
            if self.subcategory_id in [1, 3, 4, 7, 9, 10, 11, 13]:
                _idx = int(combo.get_active()) - 1
                _styles = self._dic_styles[self.subcategory_id][_idx]
            else:
                _styles = self._dic_styles[self.subcategory_id]
        except KeyError:
            _styles = []
        self.cmbStyle.do_load_combo(entries=_styles, signal="changed")

    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set widget sensitivity as needed for the selected capacitor type.

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 1:
            self.__do_set_parts_count_sensitive(attributes)
        else:
            self.__do_set_part_stress_sensitive(attributes)

    # pylint: disable=unused-argument
    def __do_set_parts_count_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set widget sensitivity as needed for MIL-HDBK-217F, Parts Count.

        :return: None
        :rtype: None
        """
        if self.subcategory_id == 1:
            self.cmbSpecification.set_sensitive(True)
        else:
            self.cmbSpecification.set_sensitive(False)
            self.cmbStyle.set_sensitive(False)
            self.cmbConfiguration.set_sensitive(False)
            self.cmbConstruction.set_sensitive(False)
            self.txtCapacitance.set_sensitive(False)
            self.txtESR.set_sensitive(False)

    def __do_set_part_stress_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set widget sensitivity as needed for MIL-HDBK-217F, Part Stress.

        :return: None
        :rtype: None
        """
        self.cmbSpecification.set_sensitive(True)
        self.cmbSpecification.do_update(
            attributes["specification_id"],
            signal="changed",
        )
        self.cmbStyle.set_sensitive(True)
        self.cmbStyle.do_update(
            attributes["type_id"],
            signal="changed",
        )
        self.txtCapacitance.set_sensitive(True)
        self.txtCapacitance.do_update(
            attributes["capacitance"],
            signal="changed",
        )

        if self.subcategory_id == 12:
            self.txtESR.set_sensitive(True)
            self.txtESR.do_update(
                attributes["resistance"],
                signal="changed",
            )
        else:
            self.txtESR.set_sensitive(False)

        if self.subcategory_id == 13:
            self.cmbConstruction.set_sensitive(True)
            self.cmbConstruction.do_update(
                attributes["construction_id"],
                signal="changed",
            )
        else:
            self.cmbConstruction.set_sensitive(False)

        if self.subcategory_id == 19:
            self.cmbConfiguration.set_sensitive(True)
            self.cmbConfiguration.do_update(
                attributes["configuration_id"],
                signal="changed",
            )
        else:
            self.cmbConfiguration.set_sensitive(False)



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.stakeholder.panel.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Stakeholder Panels."""

# Standard Library Imports
from typing import Dict, Tuple

# Third Party Imports
import treelib
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import RAMSTKTreePanel


class StakeholderTreePanel(RAMSTKTreePanel):
    """Panel to display list of stakeholder inputs."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _select_msg = "succeed_retrieve_stakeholders"
    _tag = "stakeholder"
    _title = _("Stakeholder Input List")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the stakeholder input panel."""
        super().__init__()

        # Initialize private dictionary class attributes.
        self.tvwTreeView.dic_row_loader = {
            "stakeholder": super().do_load_treerow,
        }

        # Initialize private list class attributes.

        # Initialize private scalar class attributes.
        self._on_edit_message: str = f"lvw_editing_{self._tag}"

        # Initialize public dictionary class attributes.
        self.dic_attribute_widget_map = {
            "revision_id": [
                0,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Revision ID"),
                "gint",
            ],
            "stakeholder_id": [
                1,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Stakeholder ID"),
                "gint",
            ],
            "customer_rank": [
                2,
                Gtk.CellRendererSpin(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "digits": 0,
                    "editable": True,
                    "fg_color": "#000000",
                    "lower": 1,
                    "step": 1,
                    "upper": 5,
                    "visible": True,
                },
                _("Customer Ranking"),
                "gint",
            ],
            "description": [
                3,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Description"),
                "gchararray",
            ],
            "group": [
                4,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Affinity Group"),
                "gchararray",
            ],
            "improvement": [
                5,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Improvement Factor"),
                "gfloat",
            ],
            "overall_weight": [
                6,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Overall Weighting"),
                "gfloat",
            ],
            "planned_rank": [
                7,
                Gtk.CellRendererSpin(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                1,
                {
                    "bg_color": "#FFFFFF",
                    "digits": 0,
                    "editable": True,
                    "fg_color": "#000000",
                    "lower": 1,
                    "step": 1,
                    "upper": 5,
                    "visible": True,
                },
                _("Planned Satisfaction Rating"),
                "gint",
            ],
            "priority": [
                8,
                Gtk.CellRendererSpin(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "digits": 0,
                    "editable": True,
                    "fg_color": "#000000",
                    "lower": 1,
                    "step": 1,
                    "upper": 5,
                    "visible": True,
                },
                _("Priority"),
                "gint",
            ],
            "requirement_id": [
                9,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Associated Requirement"),
                "gint",
            ],
            "stakeholder": [
                10,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Stakeholder"),
                "gchararray",
            ],
            "user_float_1": [
                11,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("User Float 1"),
                "gfloat",
            ],
            "user_float_2": [
                12,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("User Float 2"),
                "gfloat",
            ],
            "user_float_3": [
                13,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("User Float 3"),
                "gfloat",
            ],
            "user_float_4": [
                14,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("User Float 4"),
                "gfloat",
            ],
            "user_float_5": [
                15,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("User Float 5"),
                "gfloat",
            ],
        }

        # Initialize public list class attributes.

        # Initialize public scalar class attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.tvwTreeView.set_tooltip_text(_("Displays the list of stakeholders."))

        # Subscribe to PyPubSub messages.
        pub.subscribe(super().do_load_panel, "succeed_calculate_stakeholder")
        pub.subscribe(self._do_load_requirements, "succeed_retrieve_requirements")

    def do_load_affinity_groups(self, affinities: Dict[int, Tuple[str, str]]) -> None:
        """Load the affinity group list.

        :param affinities: the dict containing the affinity groups and the
            group type (affinity in all cases).
        :return: None
        """
        _cell = self.tvwTreeView.get_column(
            self.tvwTreeView.position["group"]
        ).get_cells()[0]
        _cell.set_property("has-entry", True)
        _cellmodel = _cell.get_property("model")
        _cellmodel.clear()
        _cellmodel.append([""])

        # pylint: disable=unused-variable
        for __, _key in enumerate(affinities):
            _group = affinities[_key]
            _cellmodel.append([_group[0]])

    def do_load_stakeholders(self, stakeholders: Dict[int, str]) -> None:
        """Load the stakeholder list.

        :param stakeholders: the dict containing the names of the stakeholders.
        :return: None
        """
        _cell = self.tvwTreeView.get_column(
            self.tvwTreeView.position["stakeholder"]
        ).get_cells()[0]
        _cell.set_property("has-entry", True)
        _cellmodel = _cell.get_property("model")
        _cellmodel.clear()
        _cellmodel.append([""])

        # pylint: disable=unused-variable
        for __, _key in enumerate(stakeholders):
            _group = stakeholders[_key]
            _cellmodel.append([_group])

    def _do_load_requirements(self, tree: treelib.Tree) -> None:
        """Load the requirement ID list when Requirements are retrieved.

        :param tree: the treelib Tree() containing the Stakeholder data
            records.
        :return: None
        """
        _cell = self.tvwTreeView.get_column(
            self.tvwTreeView.position["requirement_id"]
        ).get_cells()[0]
        _model = _cell.get_property("model")
        _model.clear()

        for _node in tree.nodes:
            if _node != 0:
                _model.append(
                    [str(tree.nodes[_node].data["requirement"].requirement_id)]
                )

    def _on_insert(self, tree: treelib.Tree) -> None:
        """Wrap the do_load_panel() method when an element is inserted.

        The do_set_cursor_active() method responds to the same message,
        but one less argument in it's call.  This results in a PyPubSub
        error and is the reason this wrapper method is needed.

        :param tree: the module's treelib Tree().
        :return: None
        """
        super().do_load_panel(tree)

    def _on_module_switch(self, module: str = "") -> None:
        """Respond to changes in selected Module View module (tab).

        :param module: the name of the module that was just selected.
        :return: None
        """
        _model, _row = self.tvwTreeView.selection.get_selected()

        if module == self._tag and _row is not None:
            _code = _model.get_value(_row, self.tvwTreeView.position["stakeholder_id"])
            _name = _model.get_value(_row, self.tvwTreeView.position["description"])
            _title = _(f"Analyzing Stakeholder {_code}: {_name}")

            pub.sendMessage("request_set_title", title=_title)

    def _on_row_change(self, selection: Gtk.TreeSelection) -> None:
        """Handle events for the List View RAMSTKTreeView().

        This method is called whenever a Stakeholder List View
        RAMSTKTreeView() row is activated/changed.

        :param selection: the Stakeholder class Gtk.TreeSelection().
        :return: None
        """
        _attributes = super().on_row_change(selection)

        if _attributes:
            self._record_id = _attributes["stakeholder_id"]
            self._parent_id = _attributes["requirement_id"]

            pub.sendMessage("selected_stakeholder", attributes=_attributes)



# Standard Library Imports
import gettext
from typing import Any

# RAMSTK Package Imports
from ramstk.utilities import dir_exists as dir_exists
from ramstk.utilities import file_exists as file_exists
from ramstk.utilities import get_install_prefix as get_install_prefix

_ = gettext.gettext
RAMSTK_ACTIVE_ENVIRONMENTS: Any
RAMSTK_DORMANT_ENVIRONMENTS: Any
RAMSTK_ALLOCATION_MODELS: Any
RAMSTK_HR_TYPES: Any
RAMSTK_HR_MODELS: Any
RAMSTK_HR_DISTRIBUTIONS: Any
RAMSTK_CONTROL_TYPES: Any
RAMSTK_COST_TYPES: Any
RAMSTK_MTTR_TYPES: Any
RAMSTK_CRITICALITY: Any
RAMSTK_FAILURE_PROBABILITY: Any
RAMSTK_SW_DEV_ENVIRONMENTS: Any
RAMSTK_SW_DEV_PHASES: Any
RAMSTK_SW_LEVELS: Any
RAMSTK_SW_APPLICATION: Any
RAMSTK_SW_TEST_METHODS: Any
RAMSTK_LIFECYCLE: Any
RAMSTK_S_DIST: Any

class RAMSTKSiteConfiguration:
    _INSTALL_PREFIX: Any = ...
    RAMSTK_COM_INFO: Any = ...
    RAMSTK_COM_BACKEND: str = ...
    RAMSTK_SITE_DIR: Any = ...
    RAMSTK_SITE_CONF: Any = ...
    def __init__(self) -> None: ...
    def do_create_site_configuration(self) -> None: ...
    def get_site_configuration(self) -> None: ...
    def set_site_configuration(self) -> None: ...
    def set_site_directories(self) -> None: ...

class RAMSTKUserConfiguration:
    _lst_colors: Any = ...
    _lst_format_files: Any = ...
    _lst_categories: Any = ...
    _INSTALL_PREFIX: Any = ...
    _data_sub_dir: str = ...
    _icon_sub_dir: str = ...
    _logs_sub_dir: str = ...
    RAMSTK_ACTION_CATEGORY: Any = ...
    RAMSTK_ACTION_STATUS: Any = ...
    RAMSTK_AFFINITY_GROUPS: Any = ...
    RAMSTK_CATEGORIES: Any = ...
    RAMSTK_DAMAGE_MODELS: Any = ...
    RAMSTK_DETECTION_METHODS: Any = ...
    RAMSTK_FAILURE_MODES: Any = ...
    RAMSTK_HAZARDS: Any = ...
    RAMSTK_INCIDENT_CATEGORY: Any = ...
    RAMSTK_INCIDENT_STATUS: Any = ...
    RAMSTK_INCIDENT_TYPE: Any = ...
    RAMSTK_LOAD_HISTORY: Any = ...
    RAMSTK_MANUFACTURERS: Any = ...
    RAMSTK_MEASURABLE_PARAMETERS: Any = ...
    RAMSTK_MEASUREMENT_UNITS: Any = ...
    RAMSTK_MODULES: Any = ...
    RAMSTK_REQUIREMENT_TYPE: Any = ...
    RAMSTK_RPN_DETECTION: Any = ...
    RAMSTK_RPN_OCCURRENCE: Any = ...
    RAMSTK_RPN_SEVERITY: Any = ...
    RAMSTK_SEVERITY: Any = ...
    RAMSTK_STAKEHOLDERS: Any = ...
    RAMSTK_STRESS_LIMITS: Any = ...
    RAMSTK_SUBCATEGORIES: Any = ...
    RAMSTK_USERS: Any = ...
    RAMSTK_VALIDATION_TYPE: Any = ...
    RAMSTK_COLORS: Any = ...
    RAMSTK_FORMAT_FILE: Any = ...
    RAMSTK_PAGE_NUMBER: Any = ...
    RAMSTK_PROG_INFO: Any = ...
    RAMSTK_TABPOS: Any = ...
    RAMSTK_WORKGROUPS: Any = ...
    RAMSTK_FAILURE_PROBABILITY: Any = ...
    RAMSTK_RISK_POINTS: Any = ...
    RAMSTK_MODE: str = ...
    RAMSTK_MODE_SOURCE: int = ...
    RAMSTK_BACKEND: str = ...
    RAMSTK_REPORT_SIZE: str = ...
    RAMSTK_HR_MULTIPLIER: float = ...
    RAMSTK_DEC_PLACES: int = ...
    RAMSTK_MTIME: float = ...
    RAMSTK_GUI_LAYOUT: str = ...
    RAMSTK_METHOD: str = ...
    RAMSTK_LOCALE: str = ...
    RAMSTK_LOGLEVEL: str = ...
    RAMSTK_OS: str = ...
    RAMSTK_CONF_DIR: Any = ...
    RAMSTK_HOME_DIR: Any = ...
    RAMSTK_DATA_DIR: Any = ...
    RAMSTK_ICON_DIR: Any = ...
    RAMSTK_LOG_DIR: Any = ...
    RAMSTK_PROG_DIR: Any = ...
    RAMSTK_PROG_CONF: Any = ...
    RAMSTK_USER_LOG: Any = ...
    RAMSTK_IMPORT_LOG: Any = ...
    loaded: bool = ...
    def __init__(self) -> None: ...
    def _do_make_configuration_dir(self) -> None: ...
    def _do_make_data_dir(self) -> None: ...
    def _do_make_icon_dir(self) -> None: ...
    def _do_make_log_dir(self) -> None: ...
    def _do_make_program_dir(self) -> None: ...
    def do_create_user_configuration(self) -> None: ...
    def get_user_configuration(self) -> None: ...
    def set_user_configuration(self) -> None: ...
    def set_user_directories(self) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.analyses.Allocation.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Reliability Allocation Module."""

# Standard Library Imports
from math import exp, log
from typing import Any, Dict

# Third Party Imports
from pubsub import pub


def _calculate_agree_apportionment(
    parent_goal: float, attributes: Dict[str, Any]
) -> Dict[str, Any]:
    """Perform an AGREE apportionment of a reliability requirement.

    .. note:: the AGREE method uses MTBF as the parent goal.

    :param parent_goal: the reliability goal of the parent hardware
        item.
    :param attributes: the Allocation attributes dict.
    :return: attributes; the Allocation attributes dict with updated
    values.
    :rtype: dict
    """
    _time_i: Any = (
        float(attributes["mission_time"]) * float(attributes["duty_cycle"]) / 100.0
    )
    _weight_factor: Any = float(attributes["weight_factor"])
    _n_sub_systems: Any = int(attributes["n_sub_systems"])
    _n_sub_elements: Any = int(attributes["n_sub_elements"])
    try:
        _mtbf_alloc: Any = (_n_sub_systems * _weight_factor * _time_i) / (
            _n_sub_elements * (-log(float(parent_goal)))
        )
        _hazard_rate_alloc: Any = 1.0 / _mtbf_alloc
        _reliability_alloc: Any = exp(-1.0 * _hazard_rate_alloc * _time_i)

        attributes["mtbf_alloc"] = _mtbf_alloc
        attributes["hazard_rate_alloc"] = _hazard_rate_alloc
        attributes["reliability_alloc"] = _reliability_alloc
    except ValueError:
        pub.sendMessage(
            "fail_allocate_reliability",
            error_message=(
                "Failed to apportion reliability using the "
                "AGREE method; zero or negative value "
                "passed for parent hardware item's goal.  "
                "Parent goal is {0:f}."
            ).format(parent_goal),
        )
    except ZeroDivisionError:
        pub.sendMessage(
            "fail_allocate_reliability",
            error_message=(
                "Failed to apportion reliability using "
                "the AGREE method; one or more inputs "
                "had a value of 0.0.  Subsystem "
                "mission time={0:f}, weight "
                "factor={1:f}, # of subsystems={2:d}, "
                "# of subelements={3:d}."
            ).format(_time_i, _weight_factor, _n_sub_systems, _n_sub_elements),
        )

    return attributes


def _calculate_arinc_apportionment(
    parent_goal: float, attributes: Dict[str, Any]
) -> Dict[str, Any]:
    """Perform an ARINC apportionment of the reliability requirement.

    :param parent_goal: the reliability goal of the parent hardware
        item.
    :param attributes: the Allocation attributes dict.
    :return: attributes; the Allocation attributes dict with updated
        values.
    """
    _mission_time: Any = float(attributes["mission_time"])
    _weight_factor: Any = float(attributes["weight_factor"])
    try:
        _hazard_rate_alloc: Any = _weight_factor * float(parent_goal)
        _mtbf_alloc: Any = 1.0 / _hazard_rate_alloc
        _reliability_alloc: Any = exp(-1.0 * _hazard_rate_alloc * _mission_time)
    except ZeroDivisionError:
        _mtbf_alloc = _hazard_rate_alloc = _reliability_alloc = 0.0
        pub.sendMessage(
            "fail_allocate_reliability",
            error_message=(
                "Failed to apportion reliability using "
                "the ARINC method; one or more inputs "
                "had a value of 0.0. Weight "
                "factor={0:f} and parent "
                "goal={1:f}."
            ).format(_weight_factor, parent_goal),
        )

    attributes["mtbf_alloc"] = _mtbf_alloc
    attributes["hazard_rate_alloc"] = _hazard_rate_alloc
    attributes["reliability_alloc"] = _reliability_alloc

    return attributes


def _calculate_equal_apportionment(
    parent_goal: float, attributes: Dict[str, Any]
) -> Dict[str, Any]:
    """Perform an equal apportionment of the reliability goal.

    :param parent_goal: the reliability goal of the parent hardware
        item.
    :param attributes: the Allocation attributes dict.
    :return: attributes; the Allocation attributes dict with updated
        values.
    """
    _mission_time: Any = float(attributes["mission_time"])
    _weight_factor: Any = float(attributes["weight_factor"])
    try:
        _reliability_alloc: Any = float(parent_goal) ** _weight_factor
        _hazard_rate_alloc: Any = -1.0 * log(_reliability_alloc) / _mission_time
        _mtbf_alloc: Any = 1.0 / _hazard_rate_alloc

        attributes["hazard_rate_alloc"] = _hazard_rate_alloc
        attributes["mtbf_alloc"] = _mtbf_alloc
        attributes["reliability_alloc"] = _reliability_alloc
    except TypeError:
        if parent_goal < 0.0:
            pub.sendMessage(
                "fail_allocate_reliability",
                error_message=(
                    "Failed to apportion reliability using the "
                    "equal method; a negative or zero value passed for "
                    "parent hardware item's goal.  Parent goal is "
                    "{0:f}."
                ).format(parent_goal),
            )
        else:
            raise
    except ValueError:
        pub.sendMessage(
            "fail_allocate_reliability",
            error_message=(
                "Failed to apportion reliability using the "
                "equal method; a negative or zero value passed for "
                "parent hardware item's goal.  Parent goal is "
                "{0:f}."
            ).format(parent_goal),
        )
    except ZeroDivisionError:
        pub.sendMessage(
            "fail_allocate_reliability",
            error_message=(
                "Failed to apportion reliability using the "
                "equal method; one or more inputs had a "
                "value of 0.0. Mission time={0:f} and "
                "weight factor={1:f}."
            ).format(_mission_time, _weight_factor),
        )

    return attributes


def _calculate_foo_apportionment(
    parent_goal: float, cum_weight: int, attributes: Dict[str, Any]
) -> Dict[str, Any]:
    """Perform a feasibility of objectives (FOO) apportionment.

    :param parent_goal: the failure rate requirement to allocate.
    :param cum_weight: the cumulative weight factor for all subordinate
        assemblies.
    :param attributes: the Allocation attributes dict.
    :return: attributes; the Allocation attributes dict with updated
        values.
    :rtype: dict
    """
    _intricacy: Any = int(attributes["int_factor"])
    _state_of_art: Any = int(attributes["soa_factor"])
    _operating_time: Any = int(attributes["op_time_factor"])
    _environment: Any = int(attributes["env_factor"])
    _mission_time: Any = float(attributes["mission_time"])

    try:
        _weight_factor: Any = (
            _intricacy * _state_of_art * _operating_time * _environment
        )
        _percent_weight_factor: Any = _weight_factor / int(cum_weight)

        _hazard_rate_alloc: Any = _percent_weight_factor * parent_goal
        _mtbf_alloc: Any = 1.0 / _hazard_rate_alloc
        _reliability_alloc: Any = exp(-1.0 * _hazard_rate_alloc * _mission_time)

        attributes["weight_factor"] = _weight_factor
        attributes["percent_weight_factor"] = _percent_weight_factor
        attributes["hazard_rate_alloc"] = _hazard_rate_alloc
        attributes["mtbf_alloc"] = _mtbf_alloc
        attributes["reliability_alloc"] = _reliability_alloc
    except ZeroDivisionError:
        pub.sendMessage(
            "fail_allocate_reliability",
            error_message=(
                "Failed to apportion reliability using the "
                "Feasibility of Objectives method; one or "
                "more inputs had a value of 0.0. "
                "Intricacy factor={0:d}, state of the art "
                "factor={1:d}, operating time "
                "factor={2:d}, environment factor={3:d}, "
                "cumulative weight={4:d}, parent "
                "goal={5:f}."
            ).format(
                _intricacy,
                _state_of_art,
                _operating_time,
                _environment,
                cum_weight,
                parent_goal,
            ),
        )

    return attributes


def _from_hazard_rate_goal(attributes: Dict[str, Any]) -> Dict[str, Any]:
    """Calculate the MTBF and reliability goals given the hazard rate goal.

    :param attributes: the Allocation attributes dict.
    :return: attributes; the Allocation attributes dict with updated
        values.
    :rtype: dict
    """
    _mission_time: Any = float(attributes["mission_time"])
    _hazard_rate_goal: Any = float(attributes["hazard_rate_goal"])
    try:
        _mtbf_goal: Any = 1.0 / _hazard_rate_goal
        _reliability_goal: Any = exp(-1.0 * _mission_time / _mtbf_goal)
    except ZeroDivisionError:
        _mtbf_goal = 0.0
        _reliability_goal = 0.0
        pub.sendMessage(
            "fail_calculate_allocation_goal",
            error_message=(
                "Failed to calculate the MTBF and "
                "reliability goals given the hazard rate "
                "goal.  Hazard rate goal={0:f}."
            ).format(_hazard_rate_goal),
        )

    attributes["mtbf_goal"] = _mtbf_goal
    attributes["reliability_goal"] = _reliability_goal

    return attributes


def _from_mtbf_goal(attributes: Dict[str, Any]) -> Dict[str, Any]:
    """Calculate the hazard rate and reliability goals given the MTBF goal.

    :param attributes: the Allocation attributes dict.
    :return: attributes; the Allocation attributes dict with updated
        values.
    :rtype: dict
    """
    _mission_time: Any = float(attributes["mission_time"])
    _mtbf_goal: Any = float(attributes["mtbf_goal"])
    try:
        _hazard_rate_goal: Any = 1.0 / _mtbf_goal
        _reliability_goal: Any = exp(-1.0 * _mission_time / _mtbf_goal)
    except ZeroDivisionError:
        _hazard_rate_goal = 0.0
        _reliability_goal = 0.0
        pub.sendMessage(
            "fail_calculate_allocation_goal",
            error_message=(
                "Failed to calculate the hazard rate and "
                "reliability goals given the MTBF goal.  "
                "MTBF goal={0:f}."
            ).format(_mtbf_goal),
        )

    attributes["hazard_rate_goal"] = _hazard_rate_goal
    attributes["reliability_goal"] = _reliability_goal

    return attributes


def _from_reliability_goal(attributes: Dict[str, Any]) -> Dict[str, Any]:
    """Calculate the MTBF and hazard rate goals given the reliability goal.

    :param attributes: the Allocation attributes dict.
    :return: attributes; the Allocation attributes dict with updated
        values.
    :rtype: dict
    """
    _mission_time: Any = float(attributes["mission_time"])
    _reliability_goal: Any = float(attributes["reliability_goal"])
    try:
        _mtbf_goal: Any = -1.0 * _mission_time / log(_reliability_goal)
        _hazard_rate_goal: Any = 1.0 / _mtbf_goal
    except ValueError:
        _mtbf_goal = 0.0
        _hazard_rate_goal = 0.0
        pub.sendMessage(
            "fail_calculate_allocation_goal",
            error_message=(
                "Failed to calculate the MTBF and "
                "hazard rate goals given the reliability "
                "goal.  Reliability goal={0:f}."
            ).format(_reliability_goal),
        )

    attributes["hazard_rate_goal"] = _hazard_rate_goal
    attributes["mtbf_goal"] = _mtbf_goal

    return attributes


def do_allocate_reliability(
    parent_goal: float, cumulative_weight: int, **attributes: Dict[str, Any]
) -> Dict[str, Any]:
    """Calculate the reliability allocation.

    :param parent_goal: the parent assembly's reliability goal.
    :param cumulative_weight: the cumulative weighting of all child assemblies.
        Used for feasibility of objectives method only.
    :param attributes: the Allocation attributes dict.
    :return: attributes; the Allocation attributes dict with updated values.
    :rtype: dict
    """
    if attributes["allocation_method_id"] == 1:
        attributes = _calculate_equal_apportionment(parent_goal, attributes)
    elif attributes["allocation_method_id"] == 2:
        attributes = _calculate_agree_apportionment(parent_goal, attributes)
    elif attributes["allocation_method_id"] == 3:
        attributes = _calculate_arinc_apportionment(parent_goal, attributes)
    elif attributes["allocation_method_id"] == 4:
        attributes = _calculate_foo_apportionment(
            parent_goal, cumulative_weight, attributes
        )
    else:
        _hazard_rate_alloc: Any = 0.0
        _mtbf_alloc: Any = 0.0
        _reliability_alloc: Any = 1.0
        attributes["hazard_rate_alloc"] = _hazard_rate_alloc
        attributes["mtbf_alloc"] = _mtbf_alloc
        attributes["reliability_alloc"] = _reliability_alloc

    pub.sendMessage("succeed_allocate_reliability", attributes=attributes)

    return attributes


def do_calculate_goals(**attributes: Dict[str, Any]) -> Dict[str, Any]:
    """Calculate the other two reliability metrics from the third.

    :param attributes: the Allocation attributes dict.
    :return: attributes; the Allocation attributes dict with updated values.
    :rtype: dict
    """
    if attributes["goal_measure_id"] == 1:  # Reliability goal
        attributes = _from_reliability_goal(attributes)
    elif attributes["goal_measure_id"] == 2:  # Hazard rate goal
        attributes = _from_hazard_rate_goal(attributes)
    elif attributes["goal_measure_id"] == 3:  # MTBF goal
        attributes = _from_mtbf_goal(attributes)
    else:
        _hazard_rate_goal: Any = 0.0
        _mtbf_goal: Any = 0.0
        _reliability_goal: Any = 1.0
        attributes["hazard_rate_goal"] = _hazard_rate_goal
        attributes["mtbf_goal"] = _mtbf_goal
        attributes["reliability_goal"] = _reliability_goal

    pub.sendMessage("succeed_calculate_allocation_goals", attributes=attributes)

    return attributes


def get_allocation_goal(**attributes: Dict[str, Any]) -> Dict[str, Any]:
    """Retrieve the reliability goal for the hardware item.

    Used to select the goal for the parent hardware item prior to calling
    the do_allocate_reliability() method.

    :param attributes: the selected item's Allocation attributes dict.
    :return: _goal
    :rtype: float
    :raise: KeyError if the passed attributes dict doesn't contain the
        allocation_method_id, hazard_rate_goal, and/or reliability_goal key.
    """
    if attributes["allocation_method_id"] in [2, 4]:  # ARINC or FOO.
        _goal = attributes["hazard_rate_goal"]
    else:
        _goal = attributes["reliability_goal"]

    return _goal



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.reliability.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKReliability Record Model."""

# Standard Library Imports
from math import exp
from typing import Dict, Union

# Third Party Imports
from sqlalchemy import Column, Float, ForeignKey, Integer, String

# RAMSTK Package Imports
from ramstk.analyses import dormancy
from ramstk.analyses.milhdbk217f import milhdbk217f
from ramstk.analyses.statistics import exponential, lognormal, normal, weibull
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKReliabilityRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_reliability table in RAMSTK Program database.

    This table shares a One-to-One relationship with ramstk_hardware.
    """

    __defaults__ = {
        "add_adj_factor": 0.0,
        "availability_logistics": 1.0,
        "availability_mission": 1.0,
        "avail_log_variance": 0.0,
        "avail_mis_variance": 0.0,
        "failure_distribution_id": 0,
        "hazard_rate_active": 0.0,
        "hazard_rate_dormant": 0.0,
        "hazard_rate_logistics": 0.0,
        "hazard_rate_method_id": 0,
        "hazard_rate_mission": 0.0,
        "hazard_rate_model": "",
        "hazard_rate_percent": 0.0,
        "hazard_rate_software": 0.0,
        "hazard_rate_specified": 0.0,
        "hazard_rate_type_id": 0,
        "hr_active_variance": 0.0,
        "hr_dormant_variance": 0.0,
        "hr_logistics_variance": 0.0,
        "hr_mission_variance": 0.0,
        "hr_specified_variance": 0.0,
        "lambda_b": 0.0,
        "location_parameter": 0.0,
        "mtbf_logistics": 0.0,
        "mtbf_mission": 0.0,
        "mtbf_specified": 0.0,
        "mtbf_logistics_variance": 0.0,
        "mtbf_mission_variance": 0.0,
        "mtbf_specified_variance": 0.0,
        "mult_adj_factor": 1.0,
        "quality_id": 0,
        "reliability_goal": 0.0,
        "reliability_goal_measure_id": 0,
        "reliability_logistics": 1.0,
        "reliability_mission": 1.0,
        "reliability_log_variance": 0.0,
        "reliability_miss_variance": 0.0,
        "scale_parameter": 0.0,
        "shape_parameter": 0.0,
        "survival_analysis_id": 0,
    }
    __tablename__ = "ramstk_reliability"
    __table_args__ = {"extend_existing": True}

    revision_id = Column(
        "fld_revision_id",
        Integer,
        ForeignKey("ramstk_revision.fld_revision_id", ondelete="CASCADE"),
        nullable=False,
    )
    hardware_id = Column(
        "fld_hardware_id",
        Integer,
        ForeignKey("ramstk_hardware.fld_hardware_id", ondelete="CASCADE"),
        primary_key=True,
        nullable=False,
    )

    add_adj_factor = Column(
        "fld_add_adj_factor", Float, default=__defaults__["add_adj_factor"]
    )
    availability_logistics = Column(
        "fld_availability_logistics",
        Float,
        default=__defaults__["availability_logistics"],
    )
    availability_mission = Column(
        "fld_availability_mission",
        Float,
        default=__defaults__["availability_mission"],
    )
    avail_log_variance = Column(
        "fld_avail_log_variance", Float, default=__defaults__["avail_log_variance"]
    )
    avail_mis_variance = Column(
        "fld_avail_mis_variance", Float, default=__defaults__["avail_mis_variance"]
    )
    failure_distribution_id = Column(
        "fld_failure_distribution_id",
        Integer,
        default=__defaults__["failure_distribution_id"],
    )
    hazard_rate_active = Column(
        "fld_hazard_rate_active", Float, default=__defaults__["hazard_rate_active"]
    )
    hazard_rate_dormant = Column(
        "fld_hazard_rate_dormant", Float, default=__defaults__["hazard_rate_dormant"]
    )
    hazard_rate_logistics = Column(
        "fld_hazard_rate_logistics",
        Float,
        default=__defaults__["hazard_rate_logistics"],
    )
    hazard_rate_method_id = Column(
        "fld_hazard_rate_method_id",
        Integer,
        default=__defaults__["hazard_rate_method_id"],
    )
    hazard_rate_mission = Column(
        "fld_hazard_rate_mission", Float, default=__defaults__["hazard_rate_mission"]
    )
    hazard_rate_model = Column(
        "fld_hazard_rate_model",
        String(512),
        default=__defaults__["hazard_rate_model"],
    )
    hazard_rate_percent = Column(
        "fld_hazard_rate_percent", Float, default=__defaults__["hazard_rate_percent"]
    )
    hazard_rate_software = Column(
        "fld_hazard_rate_software",
        Float,
        default=__defaults__["hazard_rate_software"],
    )
    hazard_rate_specified = Column(
        "fld_hazard_rate_specified",
        Float,
        default=__defaults__["hazard_rate_specified"],
    )
    hazard_rate_type_id = Column(
        "fld_hazard_rate_type_id", Integer, default=__defaults__["hazard_rate_type_id"]
    )
    hr_active_variance = Column(
        "fld_hr_active_variance", Float, default=__defaults__["hr_active_variance"]
    )
    hr_dormant_variance = Column(
        "fld_hr_dormant_variance", Float, default=__defaults__["hr_dormant_variance"]
    )
    hr_logistics_variance = Column(
        "fld_hr_log_variance", Float, default=__defaults__["hr_logistics_variance"]
    )
    hr_mission_variance = Column(
        "fld_hr_mis_variance", Float, default=__defaults__["hr_mission_variance"]
    )
    hr_specified_variance = Column(
        "fld_hr_spec_variance", Float, default=__defaults__["hr_specified_variance"]
    )
    lambda_b = Column("fld_lambda_b", Float, default=__defaults__["lambda_b"])
    location_parameter = Column(
        "fld_location_parameter", Float, default=__defaults__["location_parameter"]
    )
    mtbf_logistics = Column(
        "fld_mtbf_logistics", Float, default=__defaults__["mtbf_logistics"]
    )
    mtbf_mission = Column(
        "fld_mtbf_mission", Float, default=__defaults__["mtbf_mission"]
    )
    mtbf_specified = Column(
        "fld_mtbf_specified", Float, default=__defaults__["mtbf_specified"]
    )
    mtbf_logistics_variance = Column(
        "fld_mtbf_log_variance", Float, default=__defaults__["mtbf_logistics_variance"]
    )
    mtbf_mission_variance = Column(
        "fld_mtbf_mis_variance", Float, default=__defaults__["mtbf_mission_variance"]
    )
    mtbf_specified_variance = Column(
        "fld_mtbf_spec_variance", Float, default=__defaults__["mtbf_specified_variance"]
    )
    mult_adj_factor = Column(
        "fld_mult_adj_factor", Float, default=__defaults__["mult_adj_factor"]
    )
    quality_id = Column("fld_quality_id", Integer, default=__defaults__["quality_id"])
    reliability_goal = Column(
        "fld_reliability_goal", Float, default=__defaults__["reliability_goal"]
    )
    reliability_goal_measure_id = Column(
        "fld_reliability_goal_measure_id",
        Integer,
        default=__defaults__["reliability_goal_measure_id"],
    )
    reliability_logistics = Column(
        "fld_reliability_logistics",
        Float,
        default=__defaults__["reliability_logistics"],
    )
    reliability_mission = Column(
        "fld_reliability_mission", Float, default=__defaults__["reliability_mission"]
    )
    reliability_log_variance = Column(
        "fld_reliability_log_variance",
        Float,
        default=__defaults__["reliability_log_variance"],
    )
    reliability_miss_variance = Column(
        "fld_reliability_mis_variance",
        Float,
        default=__defaults__["reliability_miss_variance"],
    )
    scale_parameter = Column(
        "fld_scale_parameter", Float, default=__defaults__["scale_parameter"]
    )
    shape_parameter = Column(
        "fld_shape_parameter", Float, default=__defaults__["shape_parameter"]
    )
    survival_analysis_id = Column(
        "fld_survival_analysis_id",
        Integer,
        default=__defaults__["survival_analysis_id"],
    )

    # Define the relationships to other tables in the RAMSTK Program database.

    def get_attributes(self) -> Dict[str, Union[float, int, str]]:
        """Retrieve RAMSTKReliability attributes from RAMSTK Program database.

        :return: {hardware_id, add_adj_factor, availability_logistics,
                  availability_mission, avail_log_variance, avail_mis_variance,
                  failure_distribution_id, hazard_rate_active,
                  hazard_rate_dormant, hazard_rate_logistics,
                  hazard_rate_method_id, hazard_rate_mission,
                  hazard_rate_model, hazard_rate_percent, hazard_rate_software,
                  hazard_rate_specified, hazard_rate_type_id,
                  hr_active_variance, hr_dormant_variance,
                  hr_logistics_variance, hr_mission_variance,
                  hr_specified_variance, location_parameter, mtbf_logistics,
                  mtbf_mission, mtbf_specified, mtbf_log_variance,
                  mtbf_miss_variance, mtbf_spec_variance, mult_adj_factor,
                  quality_id, reliability_goal, reliability_goal_measure_id,
                  reliability_logistics, reliability_mission,
                  reliability_log_variance, reliability_miss_variance,
                  scale_parameter, shape_parameter, survival_analysis_id,
                  lambda_b} pairs.
        :rtype: dict
        """
        _attributes = {
            "hardware_id": self.hardware_id,
            "add_adj_factor": self.add_adj_factor,
            "availability_logistics": self.availability_logistics,
            "availability_mission": self.availability_mission,
            "avail_log_variance": self.avail_log_variance,
            "avail_mis_variance": self.avail_mis_variance,
            "failure_distribution_id": self.failure_distribution_id,
            "hazard_rate_active": self.hazard_rate_active,
            "hazard_rate_dormant": self.hazard_rate_dormant,
            "hazard_rate_logistics": self.hazard_rate_logistics,
            "hazard_rate_method_id": self.hazard_rate_method_id,
            "hazard_rate_mission": self.hazard_rate_mission,
            "hazard_rate_model": self.hazard_rate_model,
            "hazard_rate_percent": self.hazard_rate_percent,
            "hazard_rate_software": self.hazard_rate_software,
            "hazard_rate_specified": self.hazard_rate_specified,
            "hazard_rate_type_id": self.hazard_rate_type_id,
            "hr_active_variance": self.hr_active_variance,
            "hr_dormant_variance": self.hr_dormant_variance,
            "hr_logistics_variance": self.hr_logistics_variance,
            "hr_mission_variance": self.hr_mission_variance,
            "hr_specified_variance": self.hr_specified_variance,
            "lambda_b": self.lambda_b,
            "location_parameter": self.location_parameter,
            "mtbf_logistics": self.mtbf_logistics,
            "mtbf_mission": self.mtbf_mission,
            "mtbf_specified": self.mtbf_specified,
            "mtbf_logistics_variance": self.mtbf_logistics_variance,
            "mtbf_mission_variance": self.mtbf_mission_variance,
            "mtbf_specified_variance": self.mtbf_specified_variance,
            "mult_adj_factor": self.mult_adj_factor,
            "quality_id": self.quality_id,
            "reliability_goal": self.reliability_goal,
            "reliability_goal_measure_id": self.reliability_goal_measure_id,
            "reliability_logistics": self.reliability_logistics,
            "reliability_mission": self.reliability_mission,
            "reliability_log_variance": self.reliability_log_variance,
            "reliability_miss_variance": self.reliability_miss_variance,
            "scale_parameter": self.scale_parameter,
            "shape_parameter": self.shape_parameter,
            "survival_analysis_id": self.survival_analysis_id,
        }

        return _attributes

    def do_calculate_hazard_rate_active(
        self,
        multiplier: float,
        attributes: Dict[str, Union[float, int, str]],
        time: float = 1.0,
    ) -> None:
        """Calculate the active hazard rate.

        :param multiplier: the time multiplier for hazard rates.  Typically set to
            1.0 to work with failures/hour or 1000000.0 to work with failures/10^6
            hours.  Set the value in RAMSTK.toml.
        :param attributes: the aggregate attribute dict for the Hardware ID
            associated with the selected record.
        :param time: the time at which to calculate the hazard rate.  Applicable to
            non-EXP hazard functions.
        :return: _hazard_rate_active; the active hazard rate.
        :rtype: float
        """
        self.hazard_rate_active = 0.0

        if self.hazard_rate_type_id == 1:
            self.hazard_rate_active = self.do_predict_active_hazard_rate(attributes)
        elif self.hazard_rate_type_id == 2:
            self.hazard_rate_active = self.hazard_rate_specified
        elif self.hazard_rate_type_id == 3:
            self.hazard_rate_active = exponential.get_hazard_rate(self.mtbf_specified)
        elif self.hazard_rate_type_id == 4:
            _function = {
                1: exponential.get_hazard_rate(
                    self.scale_parameter,
                    location=0.0,
                ),
                2: exponential.get_hazard_rate(
                    self.scale_parameter,
                    location=self.location_parameter,
                ),
                3: lognormal.get_hazard_rate(
                    self.shape_parameter,
                    time,
                    location=0.0,
                    scale=self.scale_parameter,
                ),
                4: lognormal.get_hazard_rate(
                    self.shape_parameter,
                    time,
                    location=self.location_parameter,
                    scale=self.scale_parameter,
                ),
                5: normal.get_hazard_rate(
                    self.location_parameter,
                    self.scale_parameter,
                    time,
                ),
                6: weibull.get_hazard_rate(
                    self.shape_parameter,
                    self.scale_parameter,
                    time,
                    location=0.0,
                ),
                7: weibull.get_hazard_rate(
                    self.shape_parameter,
                    self.scale_parameter,
                    time,
                    location=self.location_parameter,
                ),
            }[self.failure_distribution_id]
            self.hazard_rate_active = _function

        self.hazard_rate_active = (
            (self.hazard_rate_active + self.add_adj_factor)
            * self.mult_adj_factor
            * (attributes["duty_cycle"] / 100.0)
            * attributes["quantity"]
            / multiplier
        )

    def do_calculate_hazard_rate_dormant(
        self,
        category_id: int,
        subcategory_id: int,
        env_active: int,
        env_dormant: int,
    ) -> None:
        """Calculate the dormant hazard rate.

        :param category_id: the ID of the component category.
        :param subcategory_id: the ID of the component subcategory.
        :param env_active: the ID of the active environment.
        :param env_dormant: the ID of the dormant environment.
        :return: _hazard_rate_dormant; the dormant hazard rate.
        :rtype: float
        """
        self.hazard_rate_dormant = dormancy.do_calculate_dormant_hazard_rate(
            [
                category_id,
                subcategory_id,
                self.hazard_rate_active,
            ],
            [
                env_active,
                env_dormant,
            ],
        )

    def do_calculate_hazard_rate_logistics(self) -> None:
        """Calculate the logistics hazard rate.

        :return: None
        :rtype: None
        """
        self.hazard_rate_logistics = (
            self.hazard_rate_active
            + self.hazard_rate_dormant
            + self.hazard_rate_software
        )

    def do_calculate_hazard_rate_mission(self, duty_cycle: float) -> None:
        """Calculate the logistics hazard rate.

        :param duty_cycle: the duty cycle of the item record.
        :return: None
        :rtype: None
        """
        self.hazard_rate_mission = (
            (self.hazard_rate_active * duty_cycle)
            + (self.hazard_rate_dormant * (1 - duty_cycle))
            + self.hazard_rate_software
        )

    def do_calculate_mtbf(self) -> None:
        """Calculate the logistics and mission MTBF.

        :return: None
        :rtype: None
        """
        try:
            self.mtbf_logistics = 1.0 / self.hazard_rate_logistics
        except ZeroDivisionError:
            self.mtbf_logistics = 0.0

        try:
            self.mtbf_mission = 1.0 / self.hazard_rate_mission
        except ZeroDivisionError:
            self.mtbf_mission = 0.0

    def do_calculate_reliability(self, time: float) -> None:
        """Calculate the reliability related metrics.

        :param time: the time at which to calculate the reliabilities.
        :return: None
        :rtype: None
        """
        self.reliability_logistics = exp(-1.0 * self.hazard_rate_logistics * time)
        self.reliability_mission = exp(-1.0 * self.hazard_rate_mission * time)

    def do_predict_active_hazard_rate(
        self, attributes: Dict[str, Union[float, int, str]]
    ) -> float:
        """Request that the hazard rate prediction be performed.

        :param attributes: the aggregate attribute dict for the Hardware ID
            associated with the record.
        :return: None
        :rtype: None
        """
        _hazard_rate_active = 0.0

        if attributes["part"] == 1 and self.hazard_rate_method_id in [1, 2]:
            _hazard_rate_active = milhdbk217f.do_predict_active_hazard_rate(
                **attributes
            )

        return _hazard_rate_active



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.RAMSTKModel.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKModel Table Module."""

# Standard Library Imports
from typing import Dict

# Third Party Imports
from sqlalchemy import Column, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKModel(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_model in the RAMSTK Common database."""

    __defaults__ = {"description": "Model Description", "model_type": 0}
    __tablename__ = "ramstk_model"
    __table_args__ = {"extend_existing": True}

    model_id = Column(
        "fld_model_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    model_type = Column("fld_model_type", Integer, default=__defaults__["model_type"])

    def get_attributes(self) -> Dict[str, object]:
        """Retrieve current values of the RAMSTKModel data model attributes.

        :return: {model_id, description, model_type} pairs.
        :rtype: dict
        """
        _attributes = {
            "model_id": self.model_id,
            "description": self.description,
            "model_type": self.model_type,
        }

        return _attributes



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.similar_item.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Similar Item Record Model."""

# Third Party Imports
# noinspection PyPackageRequirements
from sqlalchemy import Column, Float, ForeignKey, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


# pylint: disable=R0902
class RAMSTKSimilarItemRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_similar_item table in RAMSTK Program database.

    This table shares a Many-to-One relationship with ramstk_hardware.
    """

    __defaults__ = {
        "change_description_1": "",
        "change_description_2": "",
        "change_description_3": "",
        "change_description_4": "",
        "change_description_5": "",
        "change_description_6": "",
        "change_description_7": "",
        "change_description_8": "",
        "change_description_9": "",
        "change_description_10": "",
        "change_factor_1": 1.0,
        "change_factor_2": 1.0,
        "change_factor_3": 1.0,
        "change_factor_4": 1.0,
        "change_factor_5": 1.0,
        "change_factor_6": 1.0,
        "change_factor_7": 1.0,
        "change_factor_8": 1.0,
        "change_factor_9": 1.0,
        "change_factor_10": 1.0,
        "environment_from_id": 0,
        "environment_to_id": 0,
        "function_1": "0",
        "function_2": "0",
        "function_3": "0",
        "function_4": "0",
        "function_5": "0",
        "similar_item_method_id": 1,
        "parent_id": 0,
        "quality_from_id": 0,
        "quality_to_id": 0,
        "result_1": 0.0,
        "result_2": 0.0,
        "result_3": 0.0,
        "result_4": 0.0,
        "result_5": 0.0,
        "temperature_from": 30.0,
        "temperature_to": 30.0,
        "user_blob_1": "",
        "user_blob_2": "",
        "user_blob_3": "",
        "user_blob_4": "",
        "user_blob_5": "",
        "user_float_1": 0.0,
        "user_float_2": 0.0,
        "user_float_3": 0.0,
        "user_float_4": 0.0,
        "user_float_5": 0.0,
        "user_int_1": 0,
        "user_int_2": 0,
        "user_int_3": 0,
        "user_int_4": 0,
        "user_int_5": 0,
    }
    __tablename__ = "ramstk_similar_item"
    __table_args__ = {"extend_existing": True}

    revision_id = Column(
        "fld_revision_id",
        Integer,
        ForeignKey("ramstk_revision.fld_revision_id", ondelete="CASCADE"),
        nullable=False,
    )
    hardware_id = Column(
        "fld_hardware_id",
        Integer,
        ForeignKey("ramstk_hardware.fld_hardware_id", ondelete="CASCADE"),
        primary_key=True,
        nullable=False,
    )

    change_description_1 = Column(
        "fld_change_description_1", String, default=__defaults__["change_description_1"]
    )
    change_description_2 = Column(
        "fld_change_description_2", String, default=__defaults__["change_description_2"]
    )
    change_description_3 = Column(
        "fld_change_description_3", String, default=__defaults__["change_description_3"]
    )
    change_description_4 = Column(
        "fld_change_description_4", String, default=__defaults__["change_description_4"]
    )
    change_description_5 = Column(
        "fld_change_description_5", String, default=__defaults__["change_description_5"]
    )
    change_description_6 = Column(
        "fld_change_description_6", String, default=__defaults__["change_description_6"]
    )
    change_description_7 = Column(
        "fld_change_description_7", String, default=__defaults__["change_description_7"]
    )
    change_description_8 = Column(
        "fld_change_description_8", String, default=__defaults__["change_description_8"]
    )
    change_description_9 = Column(
        "fld_change_description_9", String, default=__defaults__["change_description_9"]
    )
    change_description_10 = Column(
        "fld_change_description_10",
        String,
        default=__defaults__["change_description_10"],
    )
    change_factor_1 = Column(
        "fld_change_factor_1", Float, default=__defaults__["change_factor_1"]
    )
    change_factor_2 = Column(
        "fld_change_factor_2", Float, default=__defaults__["change_factor_2"]
    )
    change_factor_3 = Column(
        "fld_change_factor_3", Float, default=__defaults__["change_factor_3"]
    )
    change_factor_4 = Column(
        "fld_change_factor_4", Float, default=__defaults__["change_factor_4"]
    )
    change_factor_5 = Column(
        "fld_change_factor_5", Float, default=__defaults__["change_factor_5"]
    )
    change_factor_6 = Column(
        "fld_change_factor_6", Float, default=__defaults__["change_factor_6"]
    )
    change_factor_7 = Column(
        "fld_change_factor_7", Float, default=__defaults__["change_factor_7"]
    )
    change_factor_8 = Column(
        "fld_change_factor_8", Float, default=__defaults__["change_factor_8"]
    )
    change_factor_9 = Column(
        "fld_change_factor_9", Float, default=__defaults__["change_factor_9"]
    )
    change_factor_10 = Column(
        "fld_change_factor_10", Float, default=__defaults__["change_factor_10"]
    )
    environment_from_id = Column(
        "fld_environment_from_id", Integer, default=__defaults__["environment_from_id"]
    )
    environment_to_id = Column(
        "fld_environment_to_id", Integer, default=__defaults__["environment_to_id"]
    )
    function_1 = Column(
        "fld_function_1", String(128), default=__defaults__["function_1"]
    )
    function_2 = Column(
        "fld_function_2", String(128), default=__defaults__["function_2"]
    )
    function_3 = Column(
        "fld_function_3", String(128), default=__defaults__["function_3"]
    )
    function_4 = Column(
        "fld_function_4", String(128), default=__defaults__["function_4"]
    )
    function_5 = Column(
        "fld_function_5", String(128), default=__defaults__["function_5"]
    )
    similar_item_method_id = Column(
        "fld_similar_item_method_id",
        Integer,
        default=__defaults__["similar_item_method_id"],
    )
    parent_id = Column("fld_parent_id", Integer, default=__defaults__["parent_id"])
    quality_from_id = Column(
        "fld_quality_from_id", Integer, default=__defaults__["quality_from_id"]
    )
    quality_to_id = Column(
        "fld_quality_to_id", Integer, default=__defaults__["quality_to_id"]
    )
    result_1 = Column("fld_result_1", Float, default=__defaults__["result_1"])
    result_2 = Column("fld_result_2", Float, default=__defaults__["result_2"])
    result_3 = Column("fld_result_3", Float, default=__defaults__["result_3"])
    result_4 = Column("fld_result_4", Float, default=__defaults__["result_4"])
    result_5 = Column("fld_result_5", Float, default=__defaults__["result_5"])
    temperature_from = Column(
        "fld_temperature_from", Float, default=__defaults__["temperature_from"]
    )
    temperature_to = Column(
        "fld_temperature_to", Float, default=__defaults__["temperature_to"]
    )
    user_blob_1 = Column("fld_user_blob_1", String, default=__defaults__["user_blob_1"])
    user_blob_2 = Column("fld_user_blob_2", String, default=__defaults__["user_blob_2"])
    user_blob_3 = Column("fld_user_blob_3", String, default=__defaults__["user_blob_3"])
    user_blob_4 = Column("fld_user_blob_4", String, default=__defaults__["user_blob_4"])
    user_blob_5 = Column("fld_user_blob_5", String, default=__defaults__["user_blob_5"])
    user_float_1 = Column(
        "fld_user_float_1", Float, default=__defaults__["user_float_1"]
    )
    user_float_2 = Column(
        "fld_user_float_2", Float, default=__defaults__["user_float_2"]
    )
    user_float_3 = Column(
        "fld_user_float_3", Float, default=__defaults__["user_float_3"]
    )
    user_float_4 = Column(
        "fld_user_float_4", Float, default=__defaults__["user_float_4"]
    )
    user_float_5 = Column(
        "fld_user_float_5", Float, default=__defaults__["user_float_5"]
    )
    user_int_1 = Column("fld_user_int_1", Integer, default=__defaults__["user_int_1"])
    user_int_2 = Column("fld_user_int_2", Integer, default=__defaults__["user_int_2"])
    user_int_3 = Column("fld_user_int_3", Integer, default=__defaults__["user_int_3"])
    user_int_4 = Column("fld_user_int_4", Integer, default=__defaults__["user_int_4"])
    user_int_5 = Column("fld_user_int_5", Integer, default=__defaults__["user_int_5"])

    # Define the relationships to other tables in the RAMSTK Program database.

    def get_attributes(self):
        """Retrieve current values of RAMSTKSimilarItem data model attributes.

        :return: {hardware_id, change_description_1, change_description_2,
                  change_description_3, change_description_4,
                  change_description_5, change_description_6,
                  change_description_7, change_description_8,
                  change_description_9, change_description_10, change_factor_1,
                  change_factor_2, change_factor_3, change_factor_4,
                  change_factor_5, change_factor_6, change_factor_7,
                  change_factor_8, change_factor_9, change_factor_10,
                  environment_from_id, environment_to_id, function_1,
                  function_2, function_3, function_4, function_5,
                  similar_item_method_id, parent_id, quality_from_id,
                  quality_to_id, result_1, result_2, result_3, result_4,
                  result_5, temperature_from, temperature_to, user_blob_1,
                  user_blob_2, user_blob_3, user_blob_4, user_blob_5,
                  user_float_1, user_float_2, user_float_3, user_float_4,
                  user_float_5, user_int_1, user_int_2, user_int_3, user_int_4,
                  user_int_5}
        :rtype: tuple
        """
        _attributes = {
            "hardware_id": self.hardware_id,
            "change_description_1": self.change_description_1,
            "change_description_2": self.change_description_2,
            "change_description_3": self.change_description_3,
            "change_description_4": self.change_description_4,
            "change_description_5": self.change_description_5,
            "change_description_6": self.change_description_6,
            "change_description_7": self.change_description_7,
            "change_description_8": self.change_description_8,
            "change_description_9": self.change_description_9,
            "change_description_10": self.change_description_10,
            "change_factor_1": self.change_factor_1,
            "change_factor_2": self.change_factor_2,
            "change_factor_3": self.change_factor_3,
            "change_factor_4": self.change_factor_4,
            "change_factor_5": self.change_factor_5,
            "change_factor_6": self.change_factor_6,
            "change_factor_7": self.change_factor_7,
            "change_factor_8": self.change_factor_8,
            "change_factor_9": self.change_factor_9,
            "change_factor_10": self.change_factor_10,
            "environment_from_id": self.environment_from_id,
            "environment_to_id": self.environment_to_id,
            "function_1": self.function_1,
            "function_2": self.function_2,
            "function_3": self.function_3,
            "function_4": self.function_4,
            "function_5": self.function_5,
            "similar_item_method_id": self.similar_item_method_id,
            "parent_id": self.parent_id,
            "quality_from_id": self.quality_from_id,
            "quality_to_id": self.quality_to_id,
            "result_1": self.result_1,
            "result_2": self.result_2,
            "result_3": self.result_3,
            "result_4": self.result_4,
            "result_5": self.result_5,
            "temperature_from": self.temperature_from,
            "temperature_to": self.temperature_to,
            "user_blob_1": self.user_blob_1,
            "user_blob_2": self.user_blob_2,
            "user_blob_3": self.user_blob_3,
            "user_blob_4": self.user_blob_4,
            "user_blob_5": self.user_blob_5,
            "user_float_1": self.user_float_1,
            "user_float_2": self.user_float_2,
            "user_float_3": self.user_float_3,
            "user_float_4": self.user_float_4,
            "user_float_5": self.user_float_5,
            "user_int_1": self.user_int_1,
            "user_int_2": self.user_int_2,
            "user_int_3": self.user_int_3,
            "user_int_4": self.user_int_4,
            "user_int_5": self.user_int_5,
        }

        return _attributes



# Standard Library Imports
from typing import Any, Callable, Dict, List, Union

# Third Party Imports
import treelib

# RAMSTK Package Imports
from ramstk.utilities import boolean_to_integer as boolean_to_integer
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _

# RAMSTK Local Imports
from .button import RAMSTKCheckButton as RAMSTKCheckButton
from .combo import RAMSTKComboBox as RAMSTKComboBox
from .entry import RAMSTKEntry as RAMSTKEntry
from .entry import RAMSTKTextView as RAMSTKTextView
from .frame import RAMSTKFrame as RAMSTKFrame
from .label import RAMSTKLabel as RAMSTKLabel
from .label import do_make_label_group as do_make_label_group
from .plot import RAMSTKPlot as RAMSTKPlot
from .scrolledwindow import RAMSTKScrolledWindow as RAMSTKScrolledWindow
from .treeview import RAMSTKTreeView as RAMSTKTreeView

class RAMSTKPanel(RAMSTKFrame):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    _dic_attribute_keys: Dict[int, List[str]]
    _dic_attribute_updater: Dict[str, List[Any]]
    _lst_labels: List[str]
    _lst_widgets: List[object]
    _parent_id: int
    _record_id: int
    _tree_loaded: bool
    dic_attribute_widget_map: Dict[str, List[Any]]
    fmt: str
    tree: treelib.Tree
    def __init__(self) -> None: ...

class RAMSTKFixedPanel(RAMSTKPanel):
    def __init__(self) -> None: ...
    def do_clear_panel(self) -> None: ...
    _record_id: int
    on_edit_callback: str
    def do_load_panel(self, attributes: Dict[str, Any]) -> None: ...
    def do_make_panel(self, **kwargs: Dict[str, Any]) -> None: ...
    def do_set_callbacks(self) -> None: ...
    def do_set_properties(self, **kwargs: Any) -> None: ...
    def on_changed_combo(
        self, combo: RAMSTKComboBox, key: str, message: str
    ) -> Dict[Union[str, Any], Any]: ...
    def on_changed_entry(
        self, entry: RAMSTKEntry, key: str, message: str
    ) -> Dict[Union[str, Any], Any]: ...
    def on_changed_textview(
        self, buffer: Gtk.TextBuffer, key: str, message: str, textview: RAMSTKTextView
    ) -> Dict[Union[str, Any], Any]: ...
    def on_edit(self, node_id: List[int], package: Dict[str, Any]) -> None: ...
    def on_toggled(
        self, checkbutton: RAMSTKCheckButton, key: str, message: str
    ) -> Dict[Union[str, Any], Any]: ...
    def __do_read_text(
        self, entry: RAMSTKEntry, keys: str, datatype: str
    ) -> Dict[str, Any]: ...

class RAMSTKPlotPanel(RAMSTKPanel):
    pltPlot: RAMSTKPlot
    def __init__(self) -> None: ...
    def do_clear_panel(self) -> None: ...
    def do_load_panel(self) -> None: ...
    def do_make_panel(self) -> None: ...
    def do_set_callbacks(self) -> None: ...
    def do_set_properties(self, **kwargs: Any) -> None: ...

class RAMSTKTreePanel(RAMSTKPanel):
    _dic_row_loader: Dict[str, Callable]
    _filtered_tree: bool
    tvwTreeView: RAMSTKTreeView
    def __init__(self) -> None: ...
    def do_clear_panel(self) -> None: ...
    def do_expand_tree(self) -> None: ...
    _record_id: int
    def do_load_row(self, attributes: Dict[str, Any]) -> None: ...
    def do_load_panel(self, tree: treelib.Tree) -> None: ...
    def do_make_panel(self) -> None: ...
    def do_make_treeview(self, **kwargs: Dict[str, Any]) -> None: ...
    def do_refresh_tree(self, node_id: List, package: Dict[str, Any]) -> None: ...
    def do_set_callbacks(self) -> None: ...
    def do_set_cell_callbacks(self, message: str, columns: List[int]) -> None: ...
    def do_set_headings(self) -> None: ...
    def do_set_properties(self, **kwargs: Any) -> None: ...
    def on_cell_edit(
        self,
        cell: Gtk.CellRenderer,
        path: str,
        new_text: str,
        position: int,
        message: str,
    ) -> None: ...
    def on_cell_toggled(
        self, cell: Gtk.CellRenderer, path: str, position: int, message: str
    ) -> None: ...
    def on_delete_treerow(self, tree: treelib.Tree) -> None: ...
    def on_insert(self, data: Any) -> None: ...
    def on_row_change(self, selection: Gtk.TreeSelection) -> Dict[str, Any]: ...
    def _do_load_row(self, node: treelib.Node, row: Gtk.TreeIter) -> Gtk.TreeIter: ...
    def _do_load_treerow(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...
